@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2004 Adrian Thurston <adriant@ragel.ca>
+ *  Copyright 2001-2005 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -26,13 +26,6 @@
 #include "bstmap.h"
 
 
-/* Init base data. */
-FGotoCodeGen::FGotoCodeGen( char *fsmName, ParseData *parseData, 
-		RedFsmAp *redFsm, std::ostream &out )
-: 
-	GotoCodeGen(fsmName, parseData, redFsm, out)
-{ }
-
 std::ostream &FGotoCodeGen::EXEC_ACTIONS()
 {
 	/* Loop the actions. */
@@ -40,18 +33,9 @@ std::ostream &FGotoCodeGen::EXEC_ACTIONS()
 		/* 	We are at the start of a glob, write the case. */
 		out << "f" << redAct->actListId << ":\n";
 
-		/* Loop the items in the list of action items. */
-		for ( ActionTable::Iter item = redAct->key; item.lte(); item++ ) {
-			/* Get the action data. */
-			Action *act = parseData->actionIndex[item->value];
-
-			/* Write the preprocessor line info for going into the 
-			 * source file. */
-			out << "#line " << act->loc.line << " \""; LDIR_PATH(inputFile) << "\"\n";
-
-			/* Wrap the block in brakets. */
-			out << "\t{"; INLINE_LIST(act->inlineList, 0, false) << "}\n";
-		}
+		/* Write each action in the list of action items. */
+		for ( ActionTable::Iter item = redAct->key; item.lte(); item++ )
+			ACTION( item->value, 0, false );
 
 		out << "\tgoto again;\n";
 	}
@@ -79,31 +63,9 @@ std::ostream &FGotoCodeGen::FINISH_CASES()
 CFGotoCodeGen::CFGotoCodeGen( char *fsmName, ParseData *parseData, 
 		RedFsmAp *redFsm, std::ostream &out )
 :
-	FGotoCodeGen(fsmName, parseData, redFsm, out)
+	FsmCodeGen(fsmName, parseData, redFsm, out)
 { }
 
-std::ostream &CFGotoCodeGen::CALL( NameInst *name, int targState, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "{fsm->_st[fsm->_top++] = _cs; _cs = " << 
-			targ->value->id << "; goto again;}";
-	return out;
-}
-
-std::ostream &CFGotoCodeGen::RET( bool inFinish )
-{
-	out << "{_cs = fsm->_st[--fsm->_top]; goto again;}";
-	return out;
-}
-
-std::ostream &CFGotoCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
-{
-	out << "{fsm->_st[fsm->_top++] = _cs; _cs = (";
-	INLINE_LIST( ilItem->children, targState, inFinish );
-	out << "); goto again;}";
-	return out;
-}
 
 void CFGotoCodeGen::writeOutCode()
 {
@@ -116,7 +78,7 @@ void CFGotoCodeGen::writeOutCode()
 
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
-		out << "	fsm->_top = 0;\n";
+		out << "	fsm->top = 0;\n";
 	
 	INIT_CODE() <<
 		"	if ( fsm->curs >= "; FIRST_FINAL_STATE() << " )\n"
@@ -227,32 +189,9 @@ void CFGotoCodeGen::writeOutCode()
 CppFGotoCodeGen::CppFGotoCodeGen( char *fsmName, ParseData *parseData, 
 		RedFsmAp *redFsm, std::ostream &out )
 :
-	FGotoCodeGen(fsmName, parseData, redFsm, out)
+	FsmCodeGen(fsmName, parseData, redFsm, out)
 { }
 
-std::ostream &CppFGotoCodeGen::CALL( NameInst *name, int targState, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "{this->_st[this->_top++] = _cs; _cs = " << 
-			targ->value->id << "; goto again;}";
-	return out;
-}
-
-std::ostream &CppFGotoCodeGen::RET( bool inFinish )
-{
-	out << "{_cs = this->_st[--this->_top]; goto again;}";
-	return out;
-}
-
-std::ostream &CppFGotoCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
-{
-	out << "{this->_st[this->_top++] = _cs; _cs = (";
-	INLINE_LIST( ilItem->children, targState, inFinish );
-	out << "); goto again;}";
-	return out;
-}
-
 void CppFGotoCodeGen::writeOutCode()
 {
 	out <<
@@ -264,7 +203,7 @@ void CppFGotoCodeGen::writeOutCode()
 
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
-		out << "	this->_top = 0;\n";
+		out << "	this->top = 0;\n";
 
 	INIT_CODE() <<
 		"	if ( this->curs >= "; FIRST_FINAL_STATE() << " )\n"
@@ -374,32 +313,9 @@ void CppFGotoCodeGen::writeOutCode()
 ObjCFGotoCodeGen::ObjCFGotoCodeGen( char *fsmName, ParseData *parseData, 
 		RedFsmAp *redFsm, std::ostream &out )
 :
-	FGotoCodeGen(fsmName, parseData, redFsm, out)
+	FsmCodeGen(fsmName, parseData, redFsm, out)
 { }
 
-std::ostream &ObjCFGotoCodeGen::CALL( NameInst *name, int targState, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "{self->_st[self->_top++] = _cs; _cs = " << 
-			targ->value->id << "; goto again;}";
-	return out;
-}
-
-std::ostream &ObjCFGotoCodeGen::RET( bool inFinish )
-{
-	out << "{_cs = self->_st[--self->_top]; goto again;}";
-	return out;
-}
-
-std::ostream &ObjCFGotoCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
-{
-	out << "{self->_st[self->_top++] = _cs; _cs = (";
-	INLINE_LIST( ilItem->children, targState, inFinish );
-	out << "); goto again;}";
-	return out;
-}
-
 void ObjCFGotoCodeGen::writeOutCode()
 {
 	out <<
@@ -407,15 +323,13 @@ void ObjCFGotoCodeGen::writeOutCode()
 		"\n";
 		
 	out <<
-		"@implementation "; FSM_NAME() << "\n\n"
-
 		"- (int)initFsm;\n"
 		"{\n"
 		"\tself->curs = "; FSM_NAME() << "_start;\n";
 
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
-		out << "\tself->_top = 0;\n";
+		out << "\tself->top = 0;\n";
 
 	INIT_CODE() <<
 		"\treturn ( self->curs >= "; FIRST_FINAL_STATE() << " ) ? 1 : 0;\n"
@@ -515,7 +429,4 @@ void ObjCFGotoCodeGen::writeOutCode()
 		CONTEXT( ctx->value->id );
 		out << "};\n\n";
 	}
-
-	out <<
-		"@end\n\n";
 }
