@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2004 Adrian Thurston <adriant@ragel.ca>
+ *  Copyright 2001-2005 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -35,65 +35,43 @@ struct ParseData;
 class FGotoCodeGen : public GotoCodeGen
 {
 public:
-	FGotoCodeGen( char *fsmName, ParseData *parseData, 
-			RedFsmAp *redFsm, std::ostream &out );
-
 	std::ostream &EXEC_ACTIONS();
 	std::ostream &FINISH_CASES();
 };
 
-
-
 /*
  * class CFGotoCodeGen
  */
-class CFGotoCodeGen : public FGotoCodeGen
+class CFGotoCodeGen : public FGotoCodeGen, public CCodeGen
 {
 public:
 	CFGotoCodeGen( char *fsmName, ParseData *parseData, 
 			RedFsmAp *redFsm, std::ostream &out );
 
-	std::ostream &CALL( NameInst *name, int targState, bool inFinish );
-	std::ostream &RET( bool inFinish );
-	std::ostream &CALLE( InlineItem *ilItem, int targState, bool inFinish );
-
-	virtual void writeOutHeader() { C_HEADER(); }
 	virtual void writeOutCode();
 };
 
-
 /*
  * class CppFGotoCodeGen
  */
-class CppFGotoCodeGen : public FGotoCodeGen
+class CppFGotoCodeGen : public FGotoCodeGen, public CppCodeGen
 {
 public:
 	CppFGotoCodeGen( char *fsmName, ParseData *parseData, 
 			RedFsmAp *redFsm, std::ostream &out );
 
-	std::ostream &CALL( NameInst *name, int targState, bool inFinish );
-	std::ostream &RET( bool inFinish );
-	std::ostream &CALLE( InlineItem *ilItem, int targState, bool inFinish );
-
-	virtual void writeOutHeader() { CPP_HEADER(); }
 	virtual void writeOutCode();
 };
 
-
 /*
  * class ObjCFGotoCodeGen
  */
-class ObjCFGotoCodeGen : public FGotoCodeGen
+class ObjCFGotoCodeGen : public FGotoCodeGen, public ObjCCodeGen
 {
 public:
 	ObjCFGotoCodeGen( char *fsmName, ParseData *parseData, 
 			RedFsmAp *redFsm, std::ostream &out );
 
-	std::ostream &CALL( NameInst *name, int targState, bool inFinish );
-	std::ostream &RET( bool inFinish );
-	std::ostream &CALLE( InlineItem *ilItem, int targState, bool inFinish );
-
-	virtual void writeOutHeader() { OBJC_HEADER(); }
 	virtual void writeOutCode();
 };
 
