@@ -1,5 +1,5 @@
 /*
- *  Copyright 2004 Adrian Thurston <adriant@ragel.ca>
+ *  Copyright 2004 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -24,13 +24,6 @@
 #include "parsetree.h"
 #include "redfsm.h"
 
-/* Pass init data to base class. */
-FlatCodeGen::FlatCodeGen( char *fsmName, ParseData *parseData, 
-		RedFsmAp *redFsm, std::ostream &out )
-:
-	FsmCodeGen(fsmName, parseData, redFsm, out)
-{ }
-
 std::ostream &FlatCodeGen::STATE_OUT_ACTION( RedStateAp *state )
 {
 	/* This function is only called if there are any out actions, so need
@@ -73,14 +66,10 @@ std::ostream &FlatCodeGen::ACTION_SWITCH()
 	for ( ActionList::Iter act = parseData->actionList; act.lte(); act++ ) {
 		/* Write out referenced actions. */
 		if ( act->numTransRefs > 0 || act->numEofRefs > 0 ) {
-			/* Get the function data. Print the case label.  */
+			/* Write the case label, the action and the case break */
 			out << "\tcase " << act->actionId << ":\n";
-
-			/* Write the preprocessor line info for going into the source file. */
-			out << "#line " << act->loc.line << " \""; LDIR_PATH(inputFile) << "\"\n";
-
-			/* Write the block and close it off. */
-			out << "\t{"; INLINE_LIST(act->inlineList, 0, false) << "} break;\n";
+			ACTION( act, 0, false );
+			out << "\tbreak;\n";
 		}
 	}
 
@@ -264,12 +253,6 @@ std::ostream &FlatCodeGen::LOCATE_TRANS()
 	return out;
 }
 
-std::ostream &FlatCodeGen::STACK( int size )
-{
-	out << "int _top, _st[" << size << "];";
-	return out;
-}
-
 std::ostream &FlatCodeGen::GOTO( NameInst *name, bool inFinish )
 {
 	/* Lookup the target. Always guaranteed to return just one target. */
@@ -320,6 +303,31 @@ std::ostream &FlatCodeGen::NEXTE( InlineItem *ilItem, bool inFinish )
 	return out;
 }
 
+std::ostream &FlatCodeGen::CALL( NameInst *name, int targState, bool inFinish )
+{
+	/* Lookup the target. Always guaranteed to return just one target. */
+	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
+	out << "{"; SELF() << "->stack["; SELF() << "->top++] = _cs; _cs = " << 
+			targ->value->id << "; goto again;}";
+	return out;
+}
+
+
+std::ostream &FlatCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
+{
+	out << "{"; SELF() << "->stack["; SELF() << "->top++] = _cs; _cs = (";
+	INLINE_LIST( ilItem->children, targState, inFinish );
+	out << "); goto again;}";
+	return out;
+}
+
+
+std::ostream &FlatCodeGen::RET( bool inFinish )
+{
+	out << "{_cs = "; SELF() << "->stack[--"; SELF() << "->top]; goto again;}";
+	return out;
+}
+
 std::ostream &FlatCodeGen::CONTEXT( int contextId )
 {
 	/* Make a vect to fill with context indexed by state. */
@@ -422,32 +430,9 @@ std::ostream &FlatCodeGen::STATE_DATA()
 CFlatCodeGen::CFlatCodeGen( char *fsmName, ParseData *parseData, 
 		RedFsmAp *redFsm, std::ostream &out )
 :
-	FlatCodeGen(fsmName, parseData, redFsm, out)
+	FsmCodeGen(fsmName, parseData, redFsm, out)
 { }
 
-std::ostream &CFlatCodeGen::CALL( NameInst *name, int targState, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "{fsm->_st[fsm->_top++] = _cs; _cs = " << 
-			targ->value->id << "; goto again;}";
-	return out;
-}
-
-std::ostream &CFlatCodeGen::RET( bool inFinish )
-{
-	out << "{_cs = fsm->_st[--fsm->_top]; goto again;}";
-	return out;
-}
-
-std::ostream &CFlatCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
-{
-	out << "{fsm->_st[fsm->_top++] = _cs; _cs = (";
-	INLINE_LIST( ilItem->children, targState, inFinish );
-	out << "); goto again;}";
-	return out;
-}
-
 void CFlatCodeGen::writeOutCode()
 {
 	/* Write the state and transition arrays. */
@@ -461,7 +446,7 @@ void CFlatCodeGen::writeOutCode()
 	
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
-		out << "	fsm->_top = 0;\n";
+		out << "	fsm->top = 0;\n";
 
 	INIT_CODE() <<
 		"	if ( fsm->curs >= "; FIRST_FINAL_STATE() << " )\n"
@@ -608,33 +593,9 @@ void CFlatCodeGen::writeOutCode()
 CppFlatCodeGen::CppFlatCodeGen( char *fsmName, ParseData *parseData, 
 		RedFsmAp *redFsm, std::ostream &out ) 
 : 
-	FlatCodeGen(fsmName, parseData, redFsm, out) 
+	FsmCodeGen(fsmName, parseData, redFsm, out) 
 { }
 
-std::ostream &CppFlatCodeGen::CALL( NameInst *name, int targState, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "{this->_st[this->_top++] = _cs; _cs = " << 
-			targ->value->id << "; goto again;}";
-	return out;
-}
-
-std::ostream &CppFlatCodeGen::RET( bool inFinish )
-{
-	out << "{_cs = this->_st[--this->_top]; goto again;}";
-	return out;
-}
-
-std::ostream &CppFlatCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	out << "{this->_st[this->_top++] = _cs; _cs = (";
-	INLINE_LIST( ilItem->children, targState, inFinish );
-	out << "); goto again;}";
-	return out;
-}
-
 void CppFlatCodeGen::writeOutCode()
 {
 	/* Write the state and transition arrays. */
@@ -648,7 +609,7 @@ void CppFlatCodeGen::writeOutCode()
 	
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
-		out << "	this->_top = 0;\n";
+		out << "	this->top = 0;\n";
 
 	INIT_CODE() <<
 		"	if ( this->curs >= "; FIRST_FINAL_STATE() << " )\n"
@@ -795,48 +756,22 @@ void CppFlatCodeGen::writeOutCode()
 ObjCFlatCodeGen::ObjCFlatCodeGen( char *fsmName, ParseData *parseData, 
 		RedFsmAp *redFsm, std::ostream &out ) 
 : 
-	FlatCodeGen(fsmName, parseData, redFsm, out) 
+	FsmCodeGen(fsmName, parseData, redFsm, out) 
 { }
 
-std::ostream &ObjCFlatCodeGen::CALL( NameInst *name, int targState, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "{self->_st[self->_top++] = _cs; _cs = " << 
-			targ->value->id << "; goto again;}";
-	return out;
-}
-
-std::ostream &ObjCFlatCodeGen::RET( bool inFinish )
-{
-	out << "{_cs = self->_st[--self->_top]; goto again;}";
-	return out;
-}
-
-std::ostream &ObjCFlatCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	out << "{self->_st[self->_top++] = _cs; _cs = (";
-	INLINE_LIST( ilItem->children, targState, inFinish );
-	out << "); goto again;}";
-	return out;
-}
-
 void ObjCFlatCodeGen::writeOutCode()
 {
 	/* Write the state and transition arrays. */
 	STATE_DATA();
 
 	out <<
-		"@implementation "; FSM_NAME() << "\n\n"
-
 		"- (int)initFsm;\n"
 		"{\n"
 		"\tself->curs = "; FSM_NAME() << "_start;\n";
 
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
-		out << "\tself->_top = 0;\n\n";
+		out << "\tself->top = 0;\n\n";
 
 	INIT_CODE() <<
 		"\treturn ( self->curs >= "; FIRST_FINAL_STATE() << " ) ? 1 : 0;\n"
@@ -966,8 +901,5 @@ void ObjCFlatCodeGen::writeOutCode()
 		CONTEXT( ctx->value->id );
 		out << "};\n\n";
 	}
-	
-	out <<
-		"@end\n\n";
 }
 
