@@ -1,5 +1,5 @@
 /*
- *  Copyright 2002-2004 Adrian Thurston <adriant@ragel.ca>
+ *  Copyright 2002-2004 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -21,30 +21,45 @@
 
 #include "fsmgraph.h"
 
-/* Insert a function into a function table. */
-void ActionTable::setAction( int ordering, int actionId )
+/* Insert an action into an action table. */
+void ActionTable::setAction( int ordering, Action *action )
 {
 	/* Multi-insert in case specific instances of an action appear in a
 	 * transition more than once. */
-	insertMulti( ordering, actionId );
+	insertMulti( ordering, action );
 }
 
-/* Set all the functions from a funcTable in this table. */
+/* Set all the action from another action table in this table. */
 void ActionTable::setActions( const ActionTable &other )
 {
 	for ( ActionTable::Iter action = other; action.lte(); action++ )
 		insertMulti( action->key, action->value );
 }
 
-void ErrActionTable::setAction( int ordering, int actionId, int transferPoint )
+/* Insert an action into an action table. */
+void LmActionTable::setAction( int ordering, LongestMatchPart *action )
 {
-	insertMulti( ErrActionTableEl( actionId, ordering, transferPoint ) );
+	/* Multi-insert in case specific instances of an action appear in a
+	 * transition more than once. */
+	insertMulti( ordering, action );
+}
+
+/* Set all the action from another action table in this table. */
+void LmActionTable::setActions( const LmActionTable &other )
+{
+	for ( LmActionTable::Iter action = other; action.lte(); action++ )
+		insertMulti( action->key, action->value );
+}
+
+void ErrActionTable::setAction( int ordering, Action *action, int transferPoint )
+{
+	insertMulti( ErrActionTableEl( action, ordering, transferPoint ) );
 }
 
 void ErrActionTable::setActions( const ErrActionTable &other )
 {
 	for ( ErrActionTable::Iter act = other; act.lte(); act++ )
-		insertMulti( ErrActionTableEl( act->actionId, act->ordering, act->transferPoint ) );
+		insertMulti( ErrActionTableEl( act->action, act->ordering, act->transferPoint ) );
 }
 
 /* Insert a priority into this priority table. Looks out for priorities on
@@ -95,12 +110,6 @@ void FsmAp::startFsmPrior( int ordering, PriorDesc *prior )
 		if ( outIt.trans->toState != 0 )
 			outIt.trans->priorTable.setPrior( ordering, prior );
 	}
-
-	/* If the new start state is final then set the out priority. This follows
-	 * the same convention as setting start funcs in the out funcs of a final
-	 * start state. */
-	if ( startState->stateBits & SB_ISFINAL )
-		startState->outPriorTable.setPrior( ordering, prior );
 }
 
 /* Set the priority of all transitions in a graph. Walks all transition lists
@@ -151,36 +160,28 @@ void FsmAp::leaveFsmPrior( int ordering, PriorDesc *prior )
  * that a machine that accepts the null string can execute a start func when it
  * matches the null word, which can only be done when leaving the start/final
  * state. */
-void FsmAp::startFsmAction( int ordering, int actionId )
+void FsmAp::startFsmAction( int ordering, Action *action )
 {
 	/* Make sure the start state has no other entry points. */
 	isolateStartState();
 
 	/* Walk the start state's transitions, setting functions. */
-	OutIter outIt( startState );
-	for ( ; ! outIt.end(); outIt++ ) {
+	for ( OutIter outIt( startState ); ! outIt.end(); outIt++ ) {
 		if ( outIt.trans->toState != 0 )
-			outIt.trans->actionTable.setAction( ordering, actionId );
+			outIt.trans->actionTable.setAction( ordering, action );
 	}
-
-	/* If start state is final then insert on the out func. This means that you
-	 * can have start and leaving funcs on a machine that recognizes the null
-	 * word. */
-	if ( startState->stateBits & SB_ISFINAL )
-		startState->outActionTable.setAction( ordering, actionId );
 }
 
 /* Set functions to execute on all transitions. Walks the out lists of all
  * states. */
-void FsmAp::allTransAction( int ordering, int actionId )
+void FsmAp::allTransAction( int ordering, Action *action )
 {
 	/* Walk all states. */
 	for ( StateList::Iter state = stateList; state.lte(); state++ ) {
 		/* Walk the out list of the state. */
-		OutIter outIt( state );
-		for ( ; ! outIt.end(); outIt++ ) {
+		for ( OutIter outIt( state ); ! outIt.end(); outIt++ ) {
 			if ( outIt.trans->toState != 0 )
-				outIt.trans->actionTable.setAction( ordering, actionId );
+				outIt.trans->actionTable.setAction( ordering, action );
 		}
 	}
 }
@@ -189,39 +190,42 @@ void FsmAp::allTransAction( int ordering, int actionId )
  * is final we can't really specify a function to execute upon entering that
  * final state the first time. So function really means whenever entering a
  * final state from within the same fsm. */
-void FsmAp::finishFsmAction( int ordering, int actionId )
+void FsmAp::finishFsmAction( int ordering, Action *action )
 {
 	/* Walk all final states. */
 	for ( StateSet::Iter state = finStateSet; state.lte(); state++ ) {
 		/* Walk the final state's in list. */
-		InIter inIt( *state );
-		for ( ; ! inIt.end(); inIt++ )
-			inIt.trans->actionTable.setAction( ordering, actionId );
+		for ( InIter inIt( *state ); ! inIt.end(); inIt++ )
+			inIt.trans->actionTable.setAction( ordering, action );
 	}
 }
 
 /* Add functions to any future out transitions that may be made going out of
  * this state machine. */
-void FsmAp::leaveFsmAction( int ordering, int actionId )
+void FsmAp::leaveFsmAction( int ordering, Action *action )
 {
 	/* Insert the action in the outActionTable of all final states. */
 	for ( StateSet::Iter state = finStateSet; state.lte(); state++ )
-		(*state)->outActionTable.setAction( ordering, actionId );
+		(*state)->outActionTable.setAction( ordering, action );
 }
 
-void FsmAp::transferOutActions( StateAp *state )
+/* Add functions to the longest match action table for constructing scanners. */
+void FsmAp::longMatchAction( int ordering, LongestMatchPart *lmPart )
 {
-	for ( ActionTable::Iter act = state->outActionTable; act.lte(); act++ )
-		state->eofActionTable.setAction( act->key, act->value );
-	state->outActionTable.empty();
+	/* Walk all final states. */
+	for ( StateSet::Iter state = finStateSet; state.lte(); state++ ) {
+		/* Walk the final state's in list. */
+		for ( InIter inIt( *state ); ! inIt.end(); inIt++ )
+			inIt.trans->lmActionTable.setAction( ordering, lmPart );
+	}
 }
 
-void FsmAp::setErrorAction( StateAp *state, int ordering, int actionId )
+void FsmAp::setErrorAction( StateAp *state, int ordering, Action *action )
 {
 	/* Set error transitions in the states that go to error. */
 	for ( TransList::Iter trans = state->outList; trans.lte(); trans++ ) {
 		if ( trans->toState == 0 )
-			trans->actionTable.setAction( ordering, actionId );
+			trans->actionTable.setAction( ordering, action );
 	}
 
 	/* If the outlist does not cover the alphabet then an error trans
@@ -231,12 +235,12 @@ void FsmAp::setErrorAction( StateAp *state, int ordering, int actionId )
 			/* There is no default transition, make one and set the
 			 * error action. */
 			TransAp *newTrans = attachNewTrans( state, 0, KeyTypeDefault, 0, 0 );
-			newTrans->actionTable.setAction( ordering, actionId );
+			newTrans->actionTable.setAction( ordering, action );
 		}
 		else if ( state->outDefault->toState == 0 ) {
 			/* There is a default that goes to the error state, add 
 			 * to the action table. */
-			state->outDefault->actionTable.setAction( ordering, actionId );
+			state->outDefault->actionTable.setAction( ordering, action );
 		}
 	}
 }
@@ -246,53 +250,52 @@ void FsmAp::transferErrorActions( StateAp *state, int transferPoint )
 	for ( int i = 0; i < state->errActionTable.length(); ) {
 		ErrActionTableEl *act = state->errActionTable.data + i;
 		if ( act->transferPoint == transferPoint ) {
-			setErrorAction( state, act->ordering, act->actionId );
-			if ( ! state->isFinState() )
-				state->eofActionTable.setAction( act->ordering, act->actionId );
-
+			/* Transfer the error action and remove it. */
+			setErrorAction( state, act->ordering, act->action );
 			state->errActionTable.BaseTable::remove( i );
 		}
 		else {
+			/* Not transfering and deleting, skip over the item. */
 			i += 1;
 		}
 	}
 }
 
 /* Set error actions in the start state. */
-void FsmAp::startErrorAction( int ordering, int actionId, int transferPoint )
+void FsmAp::startErrorAction( int ordering, Action *action, int transferPoint )
 {
 	/* Make sure the start state has no other entry points. */
 	isolateStartState();
 
 	/* Add the actions. */
-	startState->errActionTable.setAction( ordering, actionId, transferPoint );
+	startState->errActionTable.setAction( ordering, action, transferPoint );
 }
 
 /* Set error actions in all states where there is a transition out. */
-void FsmAp::allErrorAction( int ordering, int actionId, int transferPoint )
+void FsmAp::allErrorAction( int ordering, Action *action, int transferPoint )
 {
 	/* Insert actions in the error action table of all states. */
 	for ( StateList::Iter state = stateList; state.lte(); state++ )
-		state->errActionTable.setAction( ordering, actionId, transferPoint );
+		state->errActionTable.setAction( ordering, action, transferPoint );
 }
 
 /* Set error actions in the states that have transitions into a final state. */
-void FsmAp::finishErrorAction( int ordering, int actionId, int transferPoint )
+void FsmAp::finishErrorAction( int ordering, Action *action, int transferPoint )
 {
 	/* Isolate the start state in case it is reachable from in inside the
 	 * machine, in which case we don't want it set. */
 	for ( StateList::Iter state = stateList; state.lte(); state++ ) {
 		if ( state != startState && ! state->isFinState() )
-			state->errActionTable.setAction( ordering, actionId, transferPoint );
+			state->errActionTable.setAction( ordering, action, transferPoint );
 	}
 }
 
 /* Set error actions in final states. */
-void FsmAp::leaveErrorAction( int ordering, int actionId, int transferPoint )
+void FsmAp::leaveErrorAction( int ordering, Action *action, int transferPoint )
 {
 	/* Add the action to the error table of final states. */
 	for ( StateSet::Iter state = finStateSet; state.lte(); state++ )
-		(*state)->errActionTable.setAction( ordering, actionId, transferPoint );
+		(*state)->errActionTable.setAction( ordering, action, transferPoint );
 }
 
 void FsmAp::startContext( int contextId )
@@ -328,6 +331,42 @@ void FsmAp::leaveContext( int contextId )
 		(*state)->contextSet.insert( contextId );
 }
 
+/* Set EOF actions in the start state. */
+void FsmAp::startEOFAction( int ordering, Action *action )
+{
+	/* Make sure the start state has no other entry points. */
+	isolateStartState();
+
+	/* Add the actions. */
+	startState->eofActionTable.setAction( ordering, action );
+}
+
+/* Set EOF actions in all states where there is a transition out. */
+void FsmAp::allEOFAction( int ordering, Action *action )
+{
+	/* Insert actions in the EOF action table of all states. */
+	for ( StateList::Iter state = stateList; state.lte(); state++ )
+		state->eofActionTable.setAction( ordering, action );
+}
+
+/* Set EOF actions in the states that have transitions into a final state. */
+void FsmAp::finishEOFAction( int ordering, Action *action )
+{
+	/* Set the actions in all states that are not the start state and not final. */
+	for ( StateList::Iter state = stateList; state.lte(); state++ ) {
+		if ( state != startState && ! state->isFinState() )
+			state->eofActionTable.setAction( ordering, action );
+	}
+}
+
+/* Set EOF actions in final states. */
+void FsmAp::leaveEOFAction( int ordering, Action *action )
+{
+	/* Add the action to the error table of final states. */
+	for ( StateSet::Iter state = finStateSet; state.lte(); state++ )
+		(*state)->eofActionTable.setAction( ordering, action );
+}
+
 /* Shift the function ordering of the start transitions to start
  * at fromOrder and increase in units of 1. Useful before staring.
  * Returns the maximum number of order numbers used. */
@@ -378,12 +417,16 @@ void FsmAp::nullActionKeys( )
 	/* For each state... */
 	for ( StateList::Iter state = stateList; state.lte(); state++ ) {
 		/* Walk the transitions for the state. */
-		OutIter outIt( state );
-		for ( ; ! outIt.end(); outIt ++ ) {
+		for ( OutIter outIt( state ); ! outIt.end(); outIt ++ ) {
 			/* Walk the action table for the transition. */
 			for ( ActionTable::Iter action = outIt.trans->actionTable;
 					action.lte(); action++ )
 				action->key = 0;
+
+			/* Walk the action table for the transition. */
+			for ( LmActionTable::Iter action = outIt.trans->lmActionTable;
+					action.lte(); action++ )
+				action->key = 0;
 		}
 
 		/* Null the action keys of the out transtions. */
@@ -459,14 +502,20 @@ int FsmAp::comparePrior( const PriorTable &priorTable1, const PriorTable &priorT
 int FsmAp::compareTransData( TransAp *trans1, TransAp *trans2 )
 {
 	/* Compare the prior table. */
-	int cmpRes = CmpPriorTable::
-				compare( trans1->priorTable, trans2->priorTable );
+	int cmpRes = CmpPriorTable::compare( trans1->priorTable, 
+			trans2->priorTable );
+	if ( cmpRes != 0 )
+		return cmpRes;
+
+	/* Compare longest match action tables. */
+	cmpRes = CmpLmActionTable::compare(trans1->lmActionTable, 
+			trans2->lmActionTable);
 	if ( cmpRes != 0 )
 		return cmpRes;
 	
-	/* Priority tables the same. Compare function tables. */
-	return CmpActionTable::
-			compare(trans1->actionTable, trans2->actionTable);
+	/* Compare action tables. */
+	return CmpActionTable::compare(trans1->actionTable, 
+			trans2->actionTable);
 }
 
 /* Callback invoked when another trans (or possibly this) is added into this
@@ -481,10 +530,12 @@ void FsmAp::addInTrans( TransAp *destTrans, TransAp *srcTrans )
 	if ( srcTrans == destTrans ) {
 		/* Adding in ourselves, need to make a copy of the source transitions.
 		 * The priorities are not copied in as that would have no effect. */
+		destTrans->lmActionTable.setActions( LmActionTable(srcTrans->lmActionTable) );
 		destTrans->actionTable.setActions( ActionTable(srcTrans->actionTable) );
 	}
 	else {
 		/* Not a copy of ourself, get the functions and priorities. */
+		destTrans->lmActionTable.setActions( srcTrans->lmActionTable );
 		destTrans->actionTable.setActions( srcTrans->actionTable );
 		destTrans->priorTable.setPriors( srcTrans->priorTable );
 	}
