@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001 Adrian Thurston <adriant@ragel.ca>
+ *  Copyright 2001 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -195,11 +195,12 @@ void FsmAp::detachState( StateAp *state )
 		/* Detach the transitions from the source state. */
 		detachTrans( fromState, state, trans, KeyTypeRange );
 
-		/* If from has a default transition we cannot delete range from
+		/* If from has a default transition we cannot delete range
 		 * because that effectively adds the keys to the set of keys following
 		 * the default trans.  Instead null out the transition. */
 		if ( fromState->outDefault != 0 ) {
 			/* From has a default transition, we must null out the target. */
+			trans->lmActionTable.empty();
 			trans->actionTable.empty();
 			trans->priorTable.empty();
 
