@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2004 Adrian Thurston <adriant@ragel.ca>
+ *  Copyright 2001-2005 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -67,10 +67,10 @@ void FsmCodeGen::startCodeGen()
 	out << "#line " << outFilter->line + 1 << " \""; LDIR_PATH(outputFile) << "\"\n";
 }
 
-void FsmCodeGen::endCodeGen()
+void FsmCodeGen::endCodeGen( const InputLoc &loc )
 {
 	/* Write the preprocessor line info for to the input file. */
-	out << "#line " << parseData->fsmEndSecLoc.line  << " \""; LDIR_PATH(inputFile) << "\"\n";
+	out << "#line " << loc.line  << " \""; LDIR_PATH(parseData->fileName) << "\"\n";
 }
 
 /* Does the machine have any functions. */
@@ -87,42 +87,44 @@ void FsmCodeGen::findFinalActionRefs()
 		/* Rerence count out of single transitions. */
 		for ( RedTransList::Iter rtel = st->outSingle; rtel.lte(); rtel++ ) {
 			if ( rtel->value->action != 0 ) {
-				for ( ActionTable::Iter item = rtel->value->action->key; item.lte(); item++ ) {
-					Action *action = parseData->actionIndex[item->value];
-					action->numTransRefs += 1;
-				}
+				for ( ActionTable::Iter item = rtel->value->action->key; item.lte(); item++ )
+					item->value->numTransRefs += 1;
 			}
 		}
 
 		/* Reference count out of range transitions. */
 		for ( RedTransList::Iter rtel = st->outRange; rtel.lte(); rtel++ ) {
 			if ( rtel->value->action != 0 ) {
-				for ( ActionTable::Iter item = rtel->value->action->key; item.lte(); item++ ) {
-					Action *action = parseData->actionIndex[item->value];
-					action->numTransRefs += 1;
-				}
+				for ( ActionTable::Iter item = rtel->value->action->key; item.lte(); item++ )
+					item->value->numTransRefs += 1;
 			}
 		}
 
 		/* Reference count default transition. */
 		if ( st->defTrans != 0 && st->defTrans->action != 0 ) {
-			for ( ActionTable::Iter item = st->defTrans->action->key; 
-					item.lte(); item++ ) {
-				Action *action = parseData->actionIndex[item->value];
-				action->numTransRefs += 1;
-			}
+			for ( ActionTable::Iter item = st->defTrans->action->key; item.lte(); item++ )
+				item->value->numTransRefs += 1;
 		}
 
 		/* Reference count out of eof actions. */
 		if ( st->eofAction != 0 ) {
-			for ( ActionTable::Iter item = st->eofAction->key; item.lte(); item++ ) {
-				Action *action = parseData->actionIndex[item->value];
-				action->numEofRefs += 1;
-			}
+			for ( ActionTable::Iter item = st->eofAction->key; item.lte(); item++ )
+				item->value->numEofRefs += 1;
 		}
 	}
 }
 
+/* Assign ids to referenced actions. */
+void FsmCodeGen::assignActionIds()
+{
+	int nextActionId = 0;
+	for ( ActionList::Iter act = parseData->actionList; act.lte(); act++ ) {
+		/* Only ever interested in referenced actions. */
+		if ( act->numTransRefs > 0 || act->numEofRefs > 0 )
+			act->actionId = nextActionId++;
+	}
+}
+
 void FsmCodeGen::setValueLimits()
 {
 	maxSingleLen = 0;
@@ -181,8 +183,8 @@ void FsmCodeGen::setValueLimits()
 		if ( at->key.length() > maxActArrItem )
 			maxActArrItem = at->key.length();
 		for ( ActionTable::Iter item = at->key; item.lte(); item++ ) {
-			if ( item->value > maxActArrItem )
-				maxActArrItem = item->value;
+			if ( item->value->actionId > maxActArrItem )
+				maxActArrItem = item->value->actionId;
 		}
 	}
 }
@@ -271,10 +273,8 @@ void FsmCodeGen::analyzeMachine()
 
 	/* Analyze reduced action lists. */
 	for ( ActionTableMap::Iter redAct = redFsm->actionMap; redAct.lte(); redAct++ ) {
-		for ( ActionTable::Iter act = redAct->key; act.lte(); act++ ) {
-			Action *action = parseData->actionIndex[act->value];
-			analyzeActionList( redAct, action->inlineList );
-		}
+		for ( ActionTable::Iter act = redAct->key; act.lte(); act++ )
+			analyzeActionList( redAct, act->value->inlineList );
 	}
 
 	/* Find states that have transitions with actions that have next
@@ -301,6 +301,9 @@ void FsmCodeGen::analyzeMachine()
 	/* Set up the label needed flag for each state */
 	setLabelsNeeded();
 
+	/* Assign ids to actions that are referenced. */
+	assignActionIds();
+
 	/* Set the maximums of various values used for deciding types. */
 	setValueLimits();
 
@@ -329,18 +332,9 @@ std::ostream &FsmCodeGen::ARRAY_TYPE( unsigned long maxVal )
 	return out;
 }
 
-std::ostream &FsmCodeGen::C_HEADER()
+std::ostream &FsmCodeGen::C_INTERFACE()
 {
-	out << 
-		"struct "; FSM_NAME() << "\n"
-		"{\n"
-		"	/* The current state may be read and written to from outside of the\n"
-		"	 * machine.  From within action code, curs is -1 and writing to it has no\n"
-		"	 * effect. */\n"
-		"	int curs;\n";
-		STRUCT_DATA() <<
-		"};\n"
-		"\n"
+	out <<
 		"/* Initialize the machine. Invokes any init statement blocks. Returns 0\n"
 		" * if the machine begins in a non-accepting state and 1 if the machine\n"
 		" * begins in an accepting state. */\n"
@@ -357,9 +351,12 @@ std::ostream &FsmCodeGen::C_HEADER()
 		" * in the error state and does not accept, 0 if the machine finishes\n"
 		" * in any other non-accepting state and 1 if the machine finishes in an\n"
 		" * accepting state. */\n"
-		"int "; FSM_NAME() << "_finish( struct "; FSM_NAME() << " *fsm );\n"
-		"\n";
+		"int "; FSM_NAME() << "_finish( struct "; FSM_NAME() << " *fsm );\n";
+	return out;
+}
 
+std::ostream &FsmCodeGen::C_HEADER()
+{
 	for ( ContextMap::Iter ctx = parseData->contextMap; ctx.lte(); ctx++ ) {
 		out << 
 			"#define "; FSM_NAME() << "_ctx_" << ctx->key << "(state) \\\n"
@@ -372,17 +369,9 @@ std::ostream &FsmCodeGen::C_HEADER()
 }
 
 
-std::ostream &FsmCodeGen::CPP_HEADER()
+std::ostream &FsmCodeGen::CPP_INTERFACE()
 {
 	out <<
-		"class "; FSM_NAME() << " "; BASE_CLAUSE() << "\n"
-		"{\n"
-		"public:\n"
-		"	// The current state may be read and written to from outside of the\n"
-		"	// machine.  From within action code, curs is -1 and writing to it has no\n"
-		"	// effect.\n"
-		"	int curs;\n"
-		"\n"
 		"	// Initialize the machine. Invokes any init statement blocks. Returns 0\n"
 		"	// if the machine begins in a non-accepting state and 1 if the machine\n"
 		"	// begins in an accepting state.\n"
@@ -398,8 +387,7 @@ std::ostream &FsmCodeGen::CPP_HEADER()
 		"	// in the error state and does not accept, 0 if the machine finishes\n"
 		"	// in any other non-accepting state and 1 if the machine finishes in an\n"
 		"	// accepting state.\n"
-		"	int finish( );\n";
-		STRUCT_DATA() <<
+		"	int finish( );\n"
 		"\n";
 
 	for ( ContextMap::Iter ctx = parseData->contextMap; ctx.lte(); ctx++ ) {
@@ -409,30 +397,17 @@ std::ostream &FsmCodeGen::CPP_HEADER()
 			"	static unsigned char _ctxdata_" << ctx->key << "[];\n"
 			"\n";
 	}
-	
-	out <<
-		"};\n"
-		"\n";
-
-	out << "\n";
 	return out;
 }
 
+std::ostream &FsmCodeGen::CPP_HEADER()
+{
+	return out;
+}
 
-std::ostream &FsmCodeGen::OBJC_HEADER()
+std::ostream &FsmCodeGen::OBJC_INTERFACE()
 {
 	out <<
-		"@interface "; FSM_NAME() << " "; BASE_CLAUSE() << "\n"
-		"{\n"
-		"\t// The current state may be read and written to from outside of the\n"
-		"\t// machine.  From within action code, curs is -1 and writing to it has no\n"
-		"\t// effect.\n"
-		"@public\n"
-		"\tint curs;\n"
-		"\n";
-		STRUCT_DATA() <<
-		"}\n"
-		"\n"
 		"// Initialize the machine. Invokes any init statement blocks. Returns 0\n"
 		"// if the machine begins in a non-accepting state and 1 if the machine\n"
 		"// begins in an accepting state.\n"
@@ -448,11 +423,12 @@ std::ostream &FsmCodeGen::OBJC_HEADER()
 		"// in the error state and does not accept, 0 if the machine finishes\n"
 		"// in any other non-accepting state and 1 if the machine finishes in an\n"
 		"// accepting state.\n"
-		"- (int) finish;\n"
-		"\n"
-		"@end\n"
-		"\n";
+		"- (int) finish;\n";
+	return out;
+}
 
+std::ostream &FsmCodeGen::OBJC_HEADER()
+{
 	for ( ContextMap::Iter ctx = parseData->contextMap; ctx.lte(); ctx++ ) {
 		out << 
 			"#define "; FSM_NAME() << "_ctx_" << ctx->key << "(state) \\\n"
@@ -475,38 +451,6 @@ std::ostream &FsmCodeGen::FSM_NAME()
 
 std::ostream &FsmCodeGen::BASE_CLAUSE()
 {
-	if ( parseData->baseClause != 0 )
-		out << parseData->baseClause;
-	return out;
-}
-
-std::ostream &FsmCodeGen::STRUCT_DATA()
-{
-	/* Walk the list of data, printing the cases. */
-	bool anyBlocksWritten = false;
-	InlineBlock *ilBlock = parseData->dataList.head;
-	while ( ilBlock != NULL ) {
-		/* Remember that we wrote a blcok so that we know to write a line
-		 * directive back to the output file. */
-		anyBlocksWritten = true;
-
-		/* Write the preprocessor line info for going into the 
-		 * source file. */
-		out << "#line " << ilBlock->loc.line << " \"";
-		LDIR_PATH(inputFile) << "\"\n";
-
-		/* Write the block. */
-		INLINE_LIST( ilBlock->inlineList, 0, false );
-		ilBlock = ilBlock->next;
-	}
-
-	/* If any blocks were written, then write the directive for going back
-	 * into the output file. The line number is for the next line, so add one. */
-	if ( anyBlocksWritten ) {
-		out << "#line " << outFilter->line + 1 << " \"";
-		LDIR_PATH(outputFile) << "\"\n";
-	}
-
 	return out;
 }
 
@@ -532,7 +476,7 @@ std::ostream &FsmCodeGen::INIT_CODE()
 		/* Write the preprocessor line info for going into the 
 		 * source file. */
 		out << "#line " << ilBlock->loc.line << " \"";
-		LDIR_PATH(inputFile) << "\"\n";
+		LDIR_PATH(parseData->fileName) << "\"\n";
 
 		/* Write the block. */
 		out << "{";
@@ -571,7 +515,7 @@ std::ostream &FsmCodeGen::ACTIONS_ARRAY()
 			out << "\n\t";
 
 		for ( ActionTable::Iter item = act->key; item.lte(); item++ ) {
-			out << item->value;
+			out << item->value->actionId;
 			if ( ! (act.last() && item.last()) )
 				out << ", ";
 
@@ -619,7 +563,8 @@ std::ostream &FsmCodeGen::EL_TYPE()
 		INLINE_LIST( parseData->elementType, 0, false );
 	}
 	else {
-		/* No element type specified, just using alph type. */
+		/* No element type specified, just using a const alph type. */
+		out << "const ";
 		ALPH_TYPE();
 	}
 	return out;
@@ -660,6 +605,26 @@ std::ostream &FsmCodeGen::KEY( long key )
 	return out;
 }
 
+std::ostream &FsmCodeGen::LM_SWITCH( LongestMatch *longestMatch, 
+		int targState, int inFinish )
+{
+	out << "\tswitch( act ) {\n";
+
+	for ( LongestMatchList::Iter lmi = *longestMatch->longestMatchList; 
+			lmi.lte(); lmi++ )
+	{
+		if ( lmi->inLmSelect && lmi->action != 0 ) {
+			/* Write the case label, the action and the case break. */
+			out << "\tcase " << lmi->longestMatchId << ":\n";
+			ACTION( lmi->action, targState, inFinish );
+			out << "\tbreak;\n";
+		}
+	}
+
+	out << "\t}\n";
+	return out;
+}
+
 /* Write out an inline tree structure. Walks the list and possibly calls out
  * to virtual functions than handle language specific items in the tree. */
 std::ostream &FsmCodeGen::INLINE_LIST( InlineList *inlineList, int targState, bool inFinish )
@@ -682,14 +647,11 @@ std::ostream &FsmCodeGen::INLINE_LIST( InlineList *inlineList, int targState, bo
 			out << "_p";
 			break;
 		case InlineItem::Char:
-			out << "(*_p)";
+			GET_KEY();
 			break;
 		case InlineItem::Hold:
 			out << "_p--;";
 			break;
-		case InlineItem::Stack:
-			STACK(atoi(item->data));
-			break;
 		case InlineItem::Curs:
 			CURS( inFinish );
 			break;
@@ -737,6 +699,12 @@ std::ostream &FsmCodeGen::INLINE_LIST( InlineList *inlineList, int targState, bo
 			else
 				out << "_len";
 			break;
+		case InlineItem::LmSwitch:
+			LM_SWITCH( item->longestMatch, targState, inFinish );
+			break;
+		case InlineItem::LmAct:
+			INLINE_LIST( item->longestMatchPart->action->inlineList, targState, inFinish );
+			break;
 		case InlineItem::Node:
 			/* Recurse on the list of children. */
 			INLINE_LIST( item->children, targState, inFinish );
@@ -757,3 +725,31 @@ std::ostream &FsmCodeGen::LDIR_PATH( char *path )
 	}
 	return out;
 }
+
+std::ostream &FsmCodeGen::ACTION( Action *action, int targState, bool inFinish )
+{
+	/* Write the preprocessor line info for going into the source file. */
+	out << "#line " << action->loc.line << " \""; LDIR_PATH(parseData->fileName) << "\"\n";
+
+	/* Write the block and close it off. */
+	out << "\t{"; INLINE_LIST(action->inlineList, targState, inFinish) << "}\n";
+	return out;
+}
+
+std::ostream &CCodeGen::SELF()
+{
+	out << "fsm";
+	return out;
+}
+
+std::ostream &CppCodeGen::SELF()
+{
+	out << "this";
+	return out;
+}
+
+std::ostream &ObjCCodeGen::SELF()
+{
+	out << "self";
+	return out;
+}
