@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2004 Adrian Thurston <adriant@ragel.ca>
+ *  Copyright 2001-2004 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -44,6 +44,8 @@
 struct TransAp;
 struct StateAp;
 struct FsmAp;
+struct Action;
+struct LongestMatchPart;
 
 /* Differnt types of keys that transitions can be attached on. */
 enum FsmKeyType
@@ -73,8 +75,8 @@ private:
 	bool *array;
 };
 
-/* An abstraction of the key operators to enforce the possibility that the key
- * may be signed or unsigned. */
+/* An abstraction of the key operators that manages key operations such as
+ * comparison and increment according the signedness of the key. */
 struct KeyOps
 {
 	/* Default to signed alphabet. */
@@ -125,16 +127,30 @@ struct KeyOps
 };
 
 /* Transistion Action Element. */
-typedef SBstMapEl< int, int > ActionTableEl;
+typedef SBstMapEl< int, Action* > ActionTableEl;
 
 /* Transition Action Table.  */
 struct ActionTable 
-	: public SBstMap< int, int, CmpOrd<int> >
+	: public SBstMap< int, Action*, CmpOrd<int> >
 {
-	void setAction( int ordering, int actionId );
+	void setAction( int ordering, Action* action );
 	void setActions( const ActionTable &other );
 };
 
+/* The base vector of an action table. */
+typedef SVector< SBstMapEl<int, Action*> > ActionTableVect;
+
+/* Transistion Action Element. */
+typedef SBstMapEl< int, LongestMatchPart* > LmActionTableEl;
+
+/* Transition Action Table.  */
+struct LmActionTable 
+	: public SBstMap< int, LongestMatchPart*, CmpOrd<int> >
+{
+	void setAction( int ordering, LongestMatchPart* action );
+	void setActions( const LmActionTable &other );
+};
+
 /* Compare of a whole action table element (key & value). */
 struct CmpActionTableEl
 {
@@ -156,16 +172,37 @@ struct CmpActionTableEl
 /* Compare for ActionTable. */
 typedef CmpSTable< ActionTableEl, CmpActionTableEl > CmpActionTable;
 
+/* Compare of a whole lm action table element (key & value). */
+struct CmpLmActionTableEl
+{
+	static int compare( const LmActionTableEl &lmAction1, 
+			const LmActionTableEl &lmAction2 )
+	{
+		if ( lmAction1.key < lmAction2.key )
+			return -1;
+		else if ( lmAction1.key > lmAction2.key )
+			return 1;
+		else if ( lmAction1.value < lmAction2.value )
+			return -1;
+		else if ( lmAction1.value > lmAction2.value )
+			return 1;
+		return 0;
+	}
+};
+
+/* Compare for ActionTable. */
+typedef CmpSTable< LmActionTableEl, CmpLmActionTableEl > CmpLmActionTable;
+
 /* Action table element for error action tables. Adds the encoding of transfer
  * point. */
 struct ErrActionTableEl
 {
-	ErrActionTableEl( int actionId, int ordering, int transferPoint )
-		: ordering(ordering), actionId(actionId), transferPoint(transferPoint) { }
+	ErrActionTableEl( Action *action, int ordering, int transferPoint )
+		: ordering(ordering), action(action), transferPoint(transferPoint) { }
 
 	/* Ordering and id of the action embedding. */
 	int ordering;
-	int actionId;
+	Action *action;
 
 	/* Id of point of transfere from Error action table to transtions and
 	 * eofActionTable. */
@@ -179,7 +216,7 @@ struct ErrActionTable
 {
 	typedef SVector< ErrActionTableEl > BaseTable;
 
-	void setAction( int ordering, int actionId, int transferPoint );
+	void setAction( int ordering, Action *action, int transferPoint );
 	void setActions( const ErrActionTable &other );
 };
 
@@ -193,9 +230,9 @@ struct CmpErrActionTableEl
 			return -1;
 		else if ( action1.ordering > action2.ordering )
 			return 1;
-		else if ( action1.actionId < action2.actionId )
+		else if ( action1.action < action2.action )
 			return -1;
-		else if ( action1.actionId > action2.actionId )
+		else if ( action1.action > action2.action )
 			return 1;
 		else if ( action1.transferPoint < action2.transferPoint )
 			return -1;
@@ -315,6 +352,8 @@ struct TransAp
 	/* The function table and priority for the transition. */
 	ActionTable actionTable;
 	PriorTable priorTable;
+
+	LmActionTable lmActionTable;
 };
 
 /* In transition list. Like DList except only has head pointers, which is all
@@ -445,6 +484,9 @@ typedef BstSet<int> EntryIdSet;
 /* Set of context ids. */
 typedef BstSet<int> ContextSet;
 
+/* Set of longest match items that may be active in a given state. */
+typedef BstSet<LongestMatchPart*> LmItemSet;
+
 /* State class that implements actions and priorities. */
 struct StateAp 
 {
@@ -515,8 +557,8 @@ struct StateAp
 	/* 
 	 * Action and priority data.
 	 */
-
-	/* Actions to add to any transitions leaving an the via this state. */
+	
+	/* Actions to add to any future transitions that leave via this state. */
 	ActionTable outActionTable;
 
 	/* Out priorities transfered to out transitions. */
@@ -527,11 +569,14 @@ struct StateAp
 
 	/* Actions to execute on eof. */
 	ActionTable eofActionTable;
+
+	/* Set of longest match items that may be active in this state. */
+	LmItemSet lmItemSet;
 };
 
-/* Iterator for a state's out transitions.  Objects being iterated over come
- * from 3 sources: out transitions, out ranges and the default transitions.
- * Skips over unset transition elements. */
+/* Iterator for a state's out transitions. Objects being iterated over come
+ * from two sources: out ranges and the default transition. Skips key pairs
+ * that have no transition and therefore take away from the default. */
 struct OutIter
 {
 	/* Encodes the different states that an fsm iterator can be in. */
@@ -768,21 +813,21 @@ struct FsmAp
 	void leaveFsmPrior( int ordering, PriorDesc *prior );
 
 	/* Action setting support. */
-	void transferOutActions( StateAp *state );
 	void transferErrorActions( StateAp *state, int transferPoint );
-	void setErrorAction( StateAp *state, int ordering, int actionId );
+	void setErrorAction( StateAp *state, int ordering, Action *action );
 
 	/* Set actions to execute. */
-	void startFsmAction( int ordering, int actionId );
-	void allTransAction( int ordering, int actionId );
-	void finishFsmAction( int ordering, int actionId );
-	void leaveFsmAction( int ordering, int actionId );
+	void startFsmAction( int ordering, Action *action );
+	void allTransAction( int ordering, Action *action );
+	void finishFsmAction( int ordering, Action *action );
+	void leaveFsmAction( int ordering, Action *action );
+	void longMatchAction( int ordering, LongestMatchPart *lmPart );
 
 	/* Set error actions to execute. */
-	void startErrorAction( int ordering, int actionId, int transferPoint );
-	void allErrorAction( int ordering, int actionId, int transferPoint );
-	void finishErrorAction( int ordering, int actionId, int transferPoint );
-	void leaveErrorAction( int ordering, int actionId, int transferPoint );
+	void startErrorAction( int ordering, Action *action, int transferPoint );
+	void allErrorAction( int ordering, Action *action, int transferPoint );
+	void finishErrorAction( int ordering, Action *action, int transferPoint );
+	void leaveErrorAction( int ordering, Action *action, int transferPoint );
 
 	/* Set Context. */
 	void startContext( int contextId );
@@ -790,6 +835,12 @@ struct FsmAp
 	void finishContext( int contextId );
 	void leaveContext( int contextId );
 
+	/* Set EOF actions. */
+	void startEOFAction( int ordering, Action *action );
+	void allEOFAction( int ordering, Action *action );
+	void finishEOFAction( int ordering, Action *action );
+	void leaveEOFAction( int ordering, Action *action );
+
 	/* Shift the action ordering of the start transitions to start at
 	 * fromOrder and increase in units of 1. Useful before kleene star
 	 * operation.  */
@@ -1042,7 +1093,7 @@ struct FsmAp
 
 	/* Mark all states reachable from state. */
 	void markReachableFromHere( StateAp *state );
-
+	void markReachableFromHereStopFinal( StateAp *state );
 
 	/* Removes states that cannot be reached by any path in the fsm and are
 	 * thus wasted silicon. */
