@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2004 Adrian Thurston <adriant@ragel.ca>
+ *  Copyright 2001-2005 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -24,13 +24,6 @@
 #include "redfsm.h"
 #include "parsetree.h"
 
-/* Init base data. */
-FTabCodeGen::FTabCodeGen( char *fsmName, ParseData *parseData, 
-		RedFsmAp *redFsm, std::ostream &out )
-:
-	TabCodeGen(fsmName, parseData, redFsm, out)
-{ }
-
 /* Determine if we should use indicies or not. */
 void FTabCodeGen::calcIndexSize()
 {
@@ -91,18 +84,9 @@ std::ostream &FTabCodeGen::ACTION_SWITCH()
 		/* Write the entry label. */
 		out << "\tcase " << redAct->actListId+1 << ":\n";
 
-		/* Loop the items in the list of action items. */
-		for ( ActionTable::Iter item = redAct->key; item.lte(); item++ ) {
-			/* Get the action data. */
-			Action *act = parseData->actionIndex[item->value];
-
-			/* Write the preprocessor line directive for going into the source
-			 * file. */
-			out << "#line " << act->loc.line << " \""; LDIR_PATH(inputFile) << "\"\n";
-			
-			/* Wrap the block in brakets. */
-			out << "\t{"; INLINE_LIST(act->inlineList, 0, false) << "}\n";
-		}
+		/* Write each action in the list of action items. */
+		for ( ActionTable::Iter item = redAct->key; item.lte(); item++ )
+			ACTION( item->value, 0, false );
 
 		out << "\tbreak;\n";
 	}
@@ -203,32 +187,9 @@ std::ostream &FTabCodeGen::STATE_DATA()
 CFTabCodeGen::CFTabCodeGen( char *fsmName, ParseData *parseData, 
 		RedFsmAp *redFsm, std::ostream &out )
 :
-	FTabCodeGen(fsmName, parseData, redFsm, out)
+	FsmCodeGen(fsmName, parseData, redFsm, out)
 { }
 
-std::ostream &CFTabCodeGen::CALL( NameInst *name, int targState, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "{fsm->_st[fsm->_top++] = _cs; _cs = " << 
-			targ->value->id << "; goto again;}";
-	return out;
-}
-
-std::ostream &CFTabCodeGen::RET( bool inFinish )
-{
-	out << "{_cs = fsm->_st[--fsm->_top]; goto again;}";
-	return out;
-}
-
-std::ostream &CFTabCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
-{
-	out << "{fsm->_st[fsm->_top++] = _cs; _cs = (";
-	INLINE_LIST( ilItem->children, targState, inFinish );
-	out << "); goto again;}";
-	return out;
-}
-
 void CFTabCodeGen::writeOutCode()
 {
 	/* Write out arrays of states/transitions. */
@@ -242,7 +203,7 @@ void CFTabCodeGen::writeOutCode()
 
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
-		out << "	fsm->_top = 0;\n";
+		out << "	fsm->top = 0;\n";
 
 	INIT_CODE() <<
 		"	if ( fsm->curs >= "; FIRST_FINAL_STATE() << " )\n"
@@ -397,33 +358,9 @@ void CFTabCodeGen::writeOutCode()
 CppFTabCodeGen::CppFTabCodeGen( char *fsmName, ParseData *parseData, 
 		RedFsmAp *redFsm, std::ostream &out )
 :
-	FTabCodeGen(fsmName, parseData, redFsm, out) 
+	FsmCodeGen(fsmName, parseData, redFsm, out) 
 { }
 
-std::ostream &CppFTabCodeGen::CALL( NameInst *name, int targState, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "{this->_st[this->_top++] = _cs; _cs = " << 
-			targ->value->id << "; goto again;}";
-	return out;
-}
-
-std::ostream &CppFTabCodeGen::RET( bool inFinish )
-{
-	out << "{_cs = this->_st[--this->_top]; goto again;}";
-	return out;
-}
-
-std::ostream &CppFTabCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
-{
-	out << "{this->_st[this->_top++] = _cs; _cs = (";
-	INLINE_LIST( ilItem->children, targState, inFinish );
-	out << "); goto again;}";
-	return out;
-}
-
-
 void CppFTabCodeGen::writeOutCode()
 {
 	/* Write out arrays of states/transitions. */
@@ -437,7 +374,7 @@ void CppFTabCodeGen::writeOutCode()
 
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
-		out << "	this->_top = 0;\n";
+		out << "	this->top = 0;\n";
 
 	INIT_CODE() <<
 		"	if ( this->curs >= "; FIRST_FINAL_STATE() << " )\n"
@@ -590,48 +527,22 @@ void CppFTabCodeGen::writeOutCode()
 ObjCFTabCodeGen::ObjCFTabCodeGen( char *fsmName, ParseData *parseData, 
 		RedFsmAp *redFsm, std::ostream &out )
 :
-	FTabCodeGen(fsmName, parseData, redFsm, out) 
+	FsmCodeGen(fsmName, parseData, redFsm, out) 
 { }
 
-std::ostream &ObjCFTabCodeGen::CALL( NameInst *name, int targState, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "{self->_st[self->_top++] = _cs; _cs = " << 
-			targ->value->id << "; goto again;}";
-	return out;
-}
-
-std::ostream &ObjCFTabCodeGen::RET( bool inFinish )
-{
-	out << "{_cs = self->_st[--self->_top]; goto again;}";
-	return out;
-}
-
-std::ostream &ObjCFTabCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
-{
-	out << "{self->_st[self->_top++] = _cs; _cs = (";
-	INLINE_LIST( ilItem->children, targState, inFinish );
-	out << "); goto again;}";
-	return out;
-}
-
-
 void ObjCFTabCodeGen::writeOutCode()
 {
 	/* Write out arrays of states/transitions. */
 	STATE_DATA();
 
 	out <<
-		"@implementation "; FSM_NAME() << "\n\n"
-
 		"- (int)initFsm;\n"
 		"{\n"
 		"\tself->curs = "; FSM_NAME() << "_start;\n";
 
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
-		out << "\tself->_top = 0;\n\n";
+		out << "\tself->top = 0;\n\n";
 
 	INIT_CODE() <<
 		"\treturn ( self->curs >= "; FIRST_FINAL_STATE() << " ) ? 1 : 0;\n"
@@ -769,7 +680,4 @@ void ObjCFTabCodeGen::writeOutCode()
 		CONTEXT( ctx->value->id );
 		out << "};\n\n";
 	}
-	
-	out <<
-		"@end\n\n";
 }
