@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2004 Adrian Thurston <adriant@ragel.ca>
+ *  Copyright 2001-2005 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -34,10 +34,6 @@ struct ParseData;
  */
 class FTabCodeGen : public TabCodeGen
 {
-public:
-	FTabCodeGen( char *fsmName, ParseData *parseData, 
-			RedFsmAp *redFsm, std::ostream &out );
-
 protected:
 	std::ostream &ACTION_SWITCH();
 	std::ostream &STATE_DATA();
@@ -51,17 +47,12 @@ protected:
 /*
  * CFTabCodeGen
  */
-class CFTabCodeGen : public FTabCodeGen
+class CFTabCodeGen : public FTabCodeGen, public CCodeGen
 {
 public:
 	CFTabCodeGen( char *fsmName, ParseData *parseData, 
 			RedFsmAp *redFsm, std::ostream &out );
 
-	std::ostream &CALL( NameInst *name, int targState, bool inFinish );
-	std::ostream &RET( bool inFinish );
-	std::ostream &CALLE( InlineItem *ilItem, int targState, bool inFinish );
-
-	virtual void writeOutHeader() { C_HEADER(); }
 	virtual void writeOutCode();
 };
 
@@ -69,17 +60,12 @@ public:
 /*
  * class CppFTabCodeGen
  */
-class CppFTabCodeGen : public FTabCodeGen
+class CppFTabCodeGen : public FTabCodeGen, public CppCodeGen
 {
 public:
 	CppFTabCodeGen( char *fsmName, ParseData *parseData, 
 			RedFsmAp *redFsm, std::ostream &out );
 
-	std::ostream &CALL( NameInst *name, int targState, bool inFinish );
-	std::ostream &RET( bool inFinish );
-	std::ostream &CALLE( InlineItem *ilItem, int targState, bool inFinish );
-
-	virtual void writeOutHeader() { CPP_HEADER(); }
 	virtual void writeOutCode();
 };
 
@@ -87,17 +73,12 @@ public:
 /*
  * class ObjCFTabCodeGen
  */
-class ObjCFTabCodeGen : public FTabCodeGen
+class ObjCFTabCodeGen : public FTabCodeGen, public ObjCCodeGen
 {
 public:
 	ObjCFTabCodeGen( char *fsmName, ParseData *parseData, 
 			RedFsmAp *redFsm, std::ostream &out );
 
-	std::ostream &CALL( NameInst *name, int targState, bool inFinish );
-	std::ostream &RET( bool inFinish );
-	std::ostream &CALLE( InlineItem *ilItem, int targState, bool inFinish );
-
-	virtual void writeOutHeader() { OBJC_HEADER(); }
 	virtual void writeOutCode();
 };
 
