@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2004 Adrian Thurston <adriant@ragel.ca>
+ *  Copyright 2001-2005 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -25,13 +25,6 @@
 #include "parsetree.h"
 #include "bstmap.h"
 
-/* Init base data. */
-GotoCodeGen::GotoCodeGen( char *fsmName, ParseData *parseData, 
-		RedFsmAp *redFsm, std::ostream &out )
-:
-	FsmCodeGen(fsmName, parseData, redFsm, out)
-{ }
-
 /* Emit the goto to take for a given transition. */
 std::ostream &GotoCodeGen::TRANS_GOTO( RedTransAp *trans, int level )
 {
@@ -60,14 +53,10 @@ std::ostream &GotoCodeGen::ACTION_SWITCH()
 	for ( ActionList::Iter act = parseData->actionList; act.lte(); act++ ) {
 		/* Write out referenced actions. */
 		if ( act->numTransRefs > 0 || act->numEofRefs > 0 ) {
-			/* Get the function data. Print the case label.  */
+			/* Write the case label, the action and the case break. */
 			out << "\tcase " << act->actionId << ":\n";
-
-			/* Write the preprocessor line info for going into the source file. */
-			out << "#line " << act->loc.line << " \""; LDIR_PATH(inputFile) << "\"\n";
-
-			/* Write the block and close it off. */
-			out << "\t{"; INLINE_LIST(act->inlineList, 0, false) << "} break;\n";
+			ACTION( act, 0, false );
+			out << "\tbreak;\n";
 		}
 	}
 
@@ -295,13 +284,6 @@ std::ostream &GotoCodeGen::GOTOE( InlineItem *ilItem, bool inFinish )
 	return out;
 }
 
-
-std::ostream &GotoCodeGen::STACK( int size )
-{
-	out << "int _top, _st[" << size << "];";
-	return out;
-}
-
 std::ostream &GotoCodeGen::CURS( bool inFinish )
 {
 	out << "(_ps)";
@@ -336,6 +318,29 @@ std::ostream &GotoCodeGen::NEXTE( InlineItem *ilItem, bool inFinish )
 	return out;
 }
 
+std::ostream &GotoCodeGen::CALL( NameInst *name, int targState, bool inFinish )
+{
+	/* Lookup the target. Always guaranteed to return just one target. */
+	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
+	out << "{"; SELF() << "->stack["; SELF() << "->top++] = _cs; _cs = " << 
+			targ->value->id << "; goto again;}";
+	return out;
+}
+
+std::ostream &GotoCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
+{
+	out << "{"; SELF() << "->stack["; SELF() << "->top++] = _cs; _cs = (";
+	INLINE_LIST( ilItem->children, targState, inFinish );
+	out << "); goto again;}";
+	return out;
+}
+
+std::ostream &GotoCodeGen::RET( bool inFinish )
+{
+	out << "{_cs = "; SELF() << "->stack[--"; SELF() << "->top]; goto again;}";
+	return out;
+}
+
 std::ostream &GotoCodeGen::CONTEXT( int contextId )
 {
 	/* Make a vect to fill with context indexed by state. */
@@ -371,32 +376,9 @@ std::ostream &GotoCodeGen::CONTEXT( int contextId )
 CGotoCodeGen::CGotoCodeGen( char *fsmName, ParseData *parseData, 
 		RedFsmAp *redFsm, std::ostream &out ) 
 : 
-	GotoCodeGen(fsmName, parseData, redFsm, out)
+	FsmCodeGen(fsmName, parseData, redFsm, out)
 { }
 
-std::ostream &CGotoCodeGen::CALL( NameInst *name, int targState, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "{fsm->_st[fsm->_top++] = _cs; _cs = " << 
-			targ->value->id << "; goto again;}";
-	return out;
-}
-
-std::ostream &CGotoCodeGen::RET( bool inFinish )
-{
-	out << "{_cs = fsm->_st[--fsm->_top]; goto again;}";
-	return out;
-}
-
-std::ostream &CGotoCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
-{
-	out << "{fsm->_st[fsm->_top++] = _cs; _cs = (";
-	INLINE_LIST( ilItem->children, targState, inFinish );
-	out << "); goto again;}";
-	return out;
-}
-
 void CGotoCodeGen::writeOutCode()
 {
 	out <<
@@ -418,7 +400,7 @@ void CGotoCodeGen::writeOutCode()
 
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
-		out << "	fsm->_top = 0;\n";
+		out << "	fsm->top = 0;\n";
 
 	INIT_CODE() <<
 		"	if ( fsm->curs >= "; FIRST_FINAL_STATE() << " )\n"
@@ -535,31 +517,9 @@ void CGotoCodeGen::writeOutCode()
 CppGotoCodeGen::CppGotoCodeGen( char *fsmName, ParseData *parseData, 
 		RedFsmAp *redFsm, std::ostream &out )
 : 
-	GotoCodeGen(fsmName, parseData, redFsm, out)
+	FsmCodeGen(fsmName, parseData, redFsm, out)
 { }
 
-std::ostream &CppGotoCodeGen::CALL( NameInst *name, int targState, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "{this->_st[this->_top++] = _cs; _cs = " << 
-			targ->value->id << "; goto again;}";
-	return out;
-}
-
-std::ostream &CppGotoCodeGen::RET( bool inFinish )
-{
-	out << "{_cs = this->_st[--this->_top]; goto again;}";
-	return out;
-}
-
-std::ostream &CppGotoCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
-{
-	out << "{this->_st[this->_top++] = _cs; _cs = (";
-	INLINE_LIST( ilItem->children, targState, inFinish );
-	out << "); goto again;}";
-	return out;
-}
 
 void CppGotoCodeGen::writeOutCode()
 {
@@ -582,7 +542,7 @@ void CppGotoCodeGen::writeOutCode()
 
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
-		out << "	this->_top = 0;\n";
+		out << "	this->top = 0;\n";
 
 	INIT_CODE() <<
 		"	if ( this->curs >= "; FIRST_FINAL_STATE() << " )\n"
@@ -699,32 +659,9 @@ void CppGotoCodeGen::writeOutCode()
 ObjCGotoCodeGen::ObjCGotoCodeGen( char *fsmName, ParseData *parseData, 
 		RedFsmAp *redFsm, std::ostream &out )
 : 
-	GotoCodeGen(fsmName, parseData, redFsm, out)
+	FsmCodeGen(fsmName, parseData, redFsm, out)
 { }
 
-std::ostream &ObjCGotoCodeGen::CALL( NameInst *name, int targState, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "{self->_st[self->_top++] = _cs; _cs = "
-		<< targ->value->id << "; goto again;}";
-	return out;
-}
-
-std::ostream &ObjCGotoCodeGen::RET( bool inFinish )
-{
-	out << "{_cs = self->_st[--self->_top]; goto again;}";
-	return out;
-}
-
-std::ostream &ObjCGotoCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
-{
-	out << "{self->_st[self->_top++] = _cs; _cs = (";
-	INLINE_LIST( ilItem->children, targState, inFinish );
-	out << "); goto again;}";
-	return out;
-}
-
 void ObjCGotoCodeGen::writeOutCode()
 {
 	out <<
@@ -740,15 +677,13 @@ void ObjCGotoCodeGen::writeOutCode()
 	}
 
 	out <<
-		"@implementation "; FSM_NAME() << "\n\n"
-
 		"- (int)initFsm;\n"
 		"{\n"
 		"\tself->curs = "; FSM_NAME() << "_start;\n";
 
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
-		out << "\tself->_top = 0;\n\n";
+		out << "\tself->top = 0;\n\n";
 
 	INIT_CODE() <<
 		"\treturn ( self->curs >= "; FIRST_FINAL_STATE() << " ) ? 1 : 0;\n"
@@ -832,8 +767,9 @@ void ObjCGotoCodeGen::writeOutCode()
 			"\treturn [self executeWithData:0 len:1];\n";
 	}
 	else {
-		// XXX this can be optimized to take out the double self->curs read in
-		// the case where there is an errState
+		// FIXME this can be optimized to take out the double
+		// self->curs read in the case where there 
+		// is an errState
 		if ( redFsm->errState != 0 ) {
 			out << 
 			"\tif ( self->curs == "; ERROR_STATE() << " ) return -1;\n";
@@ -853,8 +789,5 @@ void ObjCGotoCodeGen::writeOutCode()
 		CONTEXT( ctx->value->id );
 		out << "};\n\n";
 	}
-	
-	out <<
-		"@end\n\n";
 }
 
