@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2004 Adrian Thurston <adriant@ragel.ca>
+ *  Copyright 2001-2005 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -34,12 +34,9 @@ struct RedStateAp;
 /*
  * Goto driven fsm.
  */
-class GotoCodeGen : public FsmCodeGen
+class GotoCodeGen : virtual public FsmCodeGen
 {
 public:
-	GotoCodeGen( char *fsmName, ParseData *parseData, 
-			RedFsmAp *redFsm, std::ostream &out );
-
 	std::ostream &FIRST_FINAL_STATE();
 	std::ostream &ERROR_STATE();
 	std::ostream &ACTION_SWITCH();
@@ -47,8 +44,6 @@ public:
 	std::ostream &TRANSITIONS();
 	std::ostream &EXEC_FUNCS();
 	std::ostream &FINISH_CASES();
-	std::ostream &STACK( int size );
-	virtual std::ostream &TRANS_GOTO( RedTransAp *trans, int level );
 	std::ostream &GOTO( NameInst *name, bool inFinish );
 	std::ostream &GOTOE( InlineItem *ilItem, bool inFinish );
 	std::ostream &CURS( bool inFinish );
@@ -56,8 +51,13 @@ public:
 	std::ostream &GET_ENTRY( NameInst *name, bool inFinish );
 	std::ostream &NEXT( NameInst *name, bool inFinish );
 	std::ostream &NEXTE( InlineItem *ilItem, bool inFinish );
+	std::ostream &CALL( NameInst *name, int targState, bool inFinish );
+	std::ostream &CALLE( InlineItem *ilItem, int targState, bool inFinish );
+	std::ostream &RET( bool inFinish );
 	std::ostream &CONTEXT( int contextId );
 
+	virtual std::ostream &TRANS_GOTO( RedTransAp *trans, int level );
+
 	void emitSingleSwitch( RedStateAp *state );
 	void emitRangeBSearch( RedStateAp *state, int level, int low, int high );
 
@@ -71,34 +71,24 @@ public:
 /*
  * class CGotoCodeGen
  */
-class CGotoCodeGen : public GotoCodeGen
+class CGotoCodeGen : public GotoCodeGen, public CCodeGen
 {
 public:
 	CGotoCodeGen( char *fsmName, ParseData *parseData, 
 			RedFsmAp *redFsm, std::ostream &out );
 
-	std::ostream &CALL( NameInst *name, int targState, bool inFinish );
-	std::ostream &RET( bool inFinish );
-	std::ostream &CALLE( InlineItem *ilItem, int targState, bool inFinish );
-
-	virtual void writeOutHeader() { C_HEADER(); }
 	virtual void writeOutCode();
 };
 
 /*
  * class CppGotoCodeGen
  */
-class CppGotoCodeGen : public GotoCodeGen
+class CppGotoCodeGen : public GotoCodeGen, public CppCodeGen
 {
 public:
 	CppGotoCodeGen( char *fsmName, ParseData *parseData, 
 			RedFsmAp *redFsm, std::ostream &out );
 
-	std::ostream &CALL( NameInst *name, int targState, bool inFinish );
-	std::ostream &RET( bool inFinish );
-	std::ostream &CALLE( InlineItem *ilItem, int targState, bool inFinish );
-
-	virtual void writeOutHeader() { CPP_HEADER(); }
 	virtual void writeOutCode();
 };
 
@@ -106,17 +96,12 @@ public:
 /*
  * class ObjCGotoCodeGen
  */
-class ObjCGotoCodeGen : public GotoCodeGen
+class ObjCGotoCodeGen : public GotoCodeGen, public ObjCCodeGen
 {
 public:
 	ObjCGotoCodeGen( char *fsmName, ParseData *parseData, 
 			RedFsmAp *redFsm, std::ostream &out );
 
-	std::ostream &CALL( NameInst *name, int targState, bool inFinish );
-	std::ostream &RET( bool inFinish );
-	std::ostream &CALLE( InlineItem *ilItem, int targState, bool inFinish );
-
-	virtual void writeOutHeader() { OBJC_HEADER(); }
 	virtual void writeOutCode();
 };
 
