@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2004 Adrian Thurston <adriant@ragel.ca>
+ *  Copyright 2001-2005 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -25,13 +25,87 @@
 #include "parsetree.h"
 #include "bstmap.h"
 
-/* Inits the code and data pointers. There are no code generation 
- * functions to register. */
-IpGotoCodeGen::IpGotoCodeGen( char *fsmName, ParseData *parseData, 
-		RedFsmAp *redFsm, std::ostream &out )
-:
-	GotoCodeGen(fsmName, parseData, redFsm, out)
-{ }
+std::ostream &IpGotoCodeGen::GOTO( NameInst *name, bool inFinish )
+{
+	/* Lookup the target. Always guaranteed to return just one target. */
+	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
+	if ( !inFinish )
+		out << "{goto st" << targ->value->id << ";}";
+	else {
+		out << "{_cs = " << targ->value->id << "; ";
+		if ( anyEofActionHold() )
+			out << "goto again;}";
+		else 
+			out << "goto out;}";
+	}
+	return out;
+}
+
+std::ostream &IpGotoCodeGen::CALL( NameInst *name, int targState, bool inFinish )
+{
+	/* Lookup the target. Always guaranteed to return just one target. */
+	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
+	out << "{"; SELF() << "->stack["; SELF() << "->top++] = " << targState << "; ";
+	if ( !inFinish )
+		out << "goto st" << targ->value->id << ";}";
+	else {
+		out << "_cs = " << targ->value->id << "; ";
+		if ( anyEofActionHold() )
+			out << "goto again;}";
+		else 
+			out << "goto out;}";
+	}
+	return out;
+}
+
+std::ostream &IpGotoCodeGen::RET( bool inFinish )
+{
+	out << "{_cs = "; SELF() << "->stack[--"; SELF() << "->top]; ";
+	if ( !inFinish || anyEofActionHold() )
+		out << "goto again;}";
+	else 
+		out << "goto out;}";
+	return out;
+}
+
+std::ostream &IpGotoCodeGen::GOTOE( InlineItem *ilItem, bool inFinish )
+{
+	out << "{_cs = (";
+	INLINE_LIST( ilItem->children, 0, inFinish );
+	out << "); ";
+	if ( !inFinish || anyEofActionHold() )
+		out << "goto again;}";
+	else 
+		out << "goto out;}";
+	return out;
+}
+
+std::ostream &IpGotoCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
+{
+	out << "{"; SELF() << "->stack["; SELF() << "->top++] = " << targState << "; _cs = (";
+	INLINE_LIST( ilItem->children, 0, inFinish );
+	out << "); ";
+	if ( !inFinish || anyEofActionHold() )
+		out << "goto again;}";
+	else 
+		out << "goto out;}";
+	return out;
+}
+
+std::ostream &IpGotoCodeGen::NEXT( NameInst *name, bool inFinish )
+{
+	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
+	out << "_cs = " << targ->value->id << ";";
+	return out;
+}
+
+std::ostream &IpGotoCodeGen::NEXTE( InlineItem *ilItem, bool inFinish )
+{
+	out << "_cs = (";
+	INLINE_LIST( ilItem->children, 0, inFinish );
+	out << ");";
+	return out;
+}
 
 /* Called from GotoCodeGen::STATE_GOTOS just before writing the gotos for each
  * state. */
@@ -55,17 +129,9 @@ std::ostream &IpGotoCodeGen::GOTO_HEADER( RedStateAp *state )
 			if ( trans->action->anyNextStmt() )
 				out << "	_cs = " << trans->targ->id << ";\n";
 
-			for ( ActionTable::Iter item = trans->action->key; item.lte(); item++ ) {
-				/* Get the function data. */
-				Action *act = parseData->actionIndex[item->value];
-
-				/* Write the preprocessor line info for going into the 
-				 * source file. */
-				out << "#line " << act->loc.line << " \""; LDIR_PATH(inputFile) << "\"\n";
-
-				/* Wrap the block in brakets. */
-				out << "\t{"; INLINE_LIST(act->inlineList, trans->targ->id, false) << "}\n";
-			}
+			/* Write each action in the list. */
+			for ( ActionTable::Iter item = trans->action->key; item.lte(); item++ )
+				ACTION( item->value, trans->targ->id, false );
 
 			/* If the action contains a next then we need to reload, otherwise
 			 * jump directly to the target state. */
@@ -136,17 +202,9 @@ std::ostream &IpGotoCodeGen::FINISH_CASES()
 			anyWritten = true;
 
 			out << "\n";
-			for ( ActionTable::Iter item = st->eofAction->key; item.lte(); item++ ) {
-				/* Get the action item. */
-				Action *act = parseData->actionIndex[item->value];
-				
-				/* Write the preprocessor line info for going into the 
-				 * source file. */
-				out << "#line " << act->loc.line << " \""; LDIR_PATH(inputFile) << "\"\n";
-
-				/* Wrap the block in brakets. */
-				out << "\t{"; INLINE_LIST(act->inlineList, STATE_ERR_STATE, true) << "}\n";
-			}
+			/* Write each action in the eof action list. */
+			for ( ActionTable::Iter item = st->eofAction->key; item.lte(); item++ )
+				ACTION( item->value, STATE_ERR_STATE, true );
 			out << "\tbreak;\n";
 		}
 	}
@@ -193,8 +251,7 @@ void IpGotoCodeGen::setLabelsNeeded()
 			/* Loop the actions. */
 			for ( ActionTable::Iter act = trans->action->key; act.lte(); act++ ) {
 				/* Get the action and walk it's tree. */
-				Action *action = parseData->actionIndex[act->value];
-				setLabelsNeeded( action->inlineList );
+				setLabelsNeeded( act->value->inlineList );
 			}
 		}
 	}
@@ -217,90 +274,9 @@ std::ostream &IpGotoCodeGen::TARGS( bool inFinish, int targState )
 CIpGotoCodeGen::CIpGotoCodeGen( char *fsmName, ParseData *parseData, 
 		RedFsmAp *redFsm, std::ostream &out )
 :
-	IpGotoCodeGen(fsmName, parseData, redFsm, out)
+	FsmCodeGen(fsmName, parseData, redFsm, out)
 { }
 
-std::ostream &CIpGotoCodeGen::GOTO( NameInst *name, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	if ( !inFinish )
-		out << "{goto st" << targ->value->id << ";}";
-	else {
-		out << "{_cs = " << targ->value->id << "; ";
-		if ( anyEofActionHold() )
-			out << "goto again;}";
-		else 
-			out << "goto out;}";
-	}
-	return out;
-}
-
-std::ostream &CIpGotoCodeGen::CALL( NameInst *name, int targState, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "{fsm->_st[fsm->_top++] = " << targState << "; ";
-	if ( !inFinish )
-		out << "goto st" << targ->value->id << ";}";
-	else {
-		out << "_cs = " << targ->value->id << "; ";
-		if ( anyEofActionHold() )
-			out << "goto again;}";
-		else 
-			out << "goto out;}";
-	}
-	return out;
-}
-
-std::ostream &CIpGotoCodeGen::RET( bool inFinish )
-{
-	out << "{_cs = fsm->_st[--fsm->_top]; ";
-	if ( !inFinish || anyEofActionHold() )
-		out << "goto again;}";
-	else 
-		out << "goto out;}";
-	return out;
-}
-
-std::ostream &CIpGotoCodeGen::GOTOE( InlineItem *ilItem, bool inFinish )
-{
-	out << "{_cs = (";
-	INLINE_LIST( ilItem->children, 0, inFinish );
-	out << "); ";
-	if ( !inFinish || anyEofActionHold() )
-		out << "goto again;}";
-	else 
-		out << "goto out;}";
-	return out;
-}
-
-std::ostream &CIpGotoCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
-{
-	out << "{fsm->_st[fsm->_top++] = " << targState << "; _cs = (";
-	INLINE_LIST( ilItem->children, targState, inFinish );
-	out << "); ";
-	if ( !inFinish || anyEofActionHold() )
-		out << "goto again;}";
-	else 
-		out << "goto out;}";
-	return out;
-}
-
-std::ostream &CIpGotoCodeGen::NEXT( NameInst *name, bool inFinish )
-{
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "_cs = " << targ->value->id << ";";
-	return out;
-}
-
-std::ostream &CIpGotoCodeGen::NEXTE( InlineItem *ilItem, bool inFinish )
-{
-	out << "_cs = (";
-	INLINE_LIST( ilItem->children, 0, inFinish );
-	out << ");";
-	return out;
-}
 
 void CIpGotoCodeGen::writeOutCode()
 {
@@ -313,7 +289,7 @@ void CIpGotoCodeGen::writeOutCode()
 
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
-		out << "	fsm->_top = 0;\n";
+		out << "	fsm->top = 0;\n";
 
 	INIT_CODE() <<
 		"	if ( fsm->curs >= "; FIRST_FINAL_STATE() << " )\n"
@@ -432,91 +408,9 @@ void CIpGotoCodeGen::writeOutCode()
 CppIpGotoCodeGen::CppIpGotoCodeGen( char *fsmName, ParseData *parseData, 
 		RedFsmAp *redFsm, std::ostream &out )
 :
-	IpGotoCodeGen(fsmName, parseData, redFsm, out)
+	FsmCodeGen(fsmName, parseData, redFsm, out)
 { }
 
-std::ostream &CppIpGotoCodeGen::GOTO( NameInst *name, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	if ( !inFinish )
-		out << "{goto st" << targ->value->id << ";}";
-	else {
-		out << "{_cs = " << targ->value->id << "; ";
-		if ( anyEofActionHold() )
-			out << "goto again;}";
-		else 
-			out << "goto out;}";
-	}
-	return out;
-}
-
-std::ostream &CppIpGotoCodeGen::CALL( NameInst *name, int targState, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "{this->_st[this->_top++] = " << targState << "; ";
-	if ( !inFinish )
-		out << "goto st" << targ->value->id << ";}";
-	else {
-		out << "_cs = " << targ->value->id << "; ";
-		if ( anyEofActionHold() )
-			out << "goto again;}";
-		else 
-			out << "goto out;}";
-	}
-	return out;
-}
-
-std::ostream &CppIpGotoCodeGen::RET( bool inFinish )
-{
-	out << "{_cs = this->_st[--this->_top]; ";
-	if ( !inFinish || anyEofActionHold() )
-		out << "goto again;}";
-	else 
-		out << "goto out;}";
-	return out;
-}
-
-std::ostream &CppIpGotoCodeGen::GOTOE( InlineItem *ilItem, bool inFinish )
-{
-	out << "{_cs = (";
-	INLINE_LIST( ilItem->children, 0, inFinish );
-	out << "); ";
-	if ( !inFinish || anyEofActionHold() )
-		out << "goto again;}";
-	else 
-		out << "goto out;}";
-	return out;
-}
-
-std::ostream &CppIpGotoCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
-{
-	out << "{this->_st[this->_top++] = " << targState << "; _cs = (";
-	INLINE_LIST( ilItem->children, 0, inFinish );
-	out << "); ";
-	if ( !inFinish || anyEofActionHold() )
-		out << "); goto again;}";
-	else 
-		out << "); goto out;}";
-	return out;
-}
-
-std::ostream &CppIpGotoCodeGen::NEXT( NameInst *name, bool inFinish )
-{
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "_cs = " << targ->value->id << ";";
-	return out;
-}
-
-std::ostream &CppIpGotoCodeGen::NEXTE( InlineItem *ilItem, bool inFinish )
-{
-	out << "_cs = (";
-	INLINE_LIST( ilItem->children, 0, inFinish );
-	out << ");";
-	return out;
-}
-
 void CppIpGotoCodeGen::writeOutCode()
 {
 	out <<
@@ -528,7 +422,7 @@ void CppIpGotoCodeGen::writeOutCode()
 
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
-		out << "	this->_top = 0;\n";
+		out << "	this->top = 0;\n";
 
 	INIT_CODE() <<
 		"	if ( this->curs >= "; FIRST_FINAL_STATE() << " )\n"
@@ -646,91 +540,9 @@ void CppIpGotoCodeGen::writeOutCode()
 ObjCIpGotoCodeGen::ObjCIpGotoCodeGen( char *fsmName, ParseData *parseData, 
 		RedFsmAp *redFsm, std::ostream &out )
 :
-	IpGotoCodeGen(fsmName, parseData, redFsm, out)
+	FsmCodeGen(fsmName, parseData, redFsm, out)
 { }
 
-std::ostream &ObjCIpGotoCodeGen::GOTO( NameInst *name, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	if ( !inFinish )
-		out << "{goto st" << targ->value->id << ";}";
-	else {
-		out << "{_cs = " << targ->value->id << "; ";
-		if ( anyEofActionHold() )
-			out << "goto again;}";
-		else 
-			out << "goto out;}";
-	}
-	return out;
-}
-
-std::ostream &ObjCIpGotoCodeGen::CALL( NameInst *name, int targState, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "{self->_st[self->_top++] = " << targState << "; ";
-	if ( !inFinish )
-		out << "goto st" << targ->value->id << ";}";
-	else {
-		out << "_cs = " << targ->value->id << "; ";
-		if ( anyEofActionHold() )
-			out << "goto again;}";
-		else 
-			out << "goto out;}";
-	}
-	return out;
-}
-
-std::ostream &ObjCIpGotoCodeGen::RET( bool inFinish )
-{
-	out << "{_cs = self->_st[--self->_top]; ";
-	if ( !inFinish || anyEofActionHold() )
-		out << "goto again;}";
-	else 
-		out << "goto out;}";
-	return out;
-}
-
-std::ostream &ObjCIpGotoCodeGen::GOTOE( InlineItem *ilItem, bool inFinish )
-{
-	out << "{_cs = (";
-	INLINE_LIST( ilItem->children, 0, inFinish );
-	out << "); ";
-	if ( !inFinish || anyEofActionHold() )
-		out << "goto again;}";
-	else 
-		out << "goto out;}";
-	return out;
-}
-
-std::ostream &ObjCIpGotoCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
-{
-	out << "{self->_st[self->_top++] = " << targState << "; _cs = (";
-	INLINE_LIST( ilItem->children, targState, inFinish );
-	out << "); ";
-	if ( !inFinish || anyEofActionHold() )
-		out << "); goto again;}";
-	else 
-		out << "); goto out;}";
-	return out;
-}
-
-std::ostream &ObjCIpGotoCodeGen::NEXT( NameInst *name, bool inFinish )
-{
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "_cs = " << targ->value->id << ";";
-	return out;
-}
-
-std::ostream &ObjCIpGotoCodeGen::NEXTE( InlineItem *ilItem, bool inFinish )
-{
-	out << "_cs = (";
-	INLINE_LIST( ilItem->children, 0, inFinish );
-	out << ");";
-	return out;
-}
-
 void ObjCIpGotoCodeGen::writeOutCode()
 {
 	out <<
@@ -738,15 +550,13 @@ void ObjCIpGotoCodeGen::writeOutCode()
 		"\n";
 		
 	out <<
-		"@implementation "; FSM_NAME() << "\n\n"
-
 		"- (int)initFsm;\n"
 		"{\n"
 		"\tself->curs = "; FSM_NAME() << "_start;\n";
 
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
-		out << "\tself->_top = 0;\n";
+		out << "\tself->top = 0;\n";
 
 	INIT_CODE() <<
 			"\treturn ( self->curs >= "; FIRST_FINAL_STATE() << " ) ? 1 : 0;\n"
@@ -855,7 +665,4 @@ void ObjCIpGotoCodeGen::writeOutCode()
 		CONTEXT( ctx->value->id );
 		out << "};\n\n";
 	}
-
-	out <<
-		"@end\n\n";
 }
