@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2004 Adrian Thurston <adriant@ragel.ca>
+ *  Copyright 2001-2005 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -34,14 +34,19 @@ struct ParseData;
 class IpGotoCodeGen : public GotoCodeGen
 {
 public:
-	IpGotoCodeGen( char *fsmName, ParseData *parseData, 
-			RedFsmAp *redFsm, std::ostream &out );
-
 	std::ostream &EXIT_STATES();
 	std::ostream &TRANS_GOTO( RedTransAp *trans, int level );
 	std::ostream &FINISH_CASES();
 	std::ostream &CURS( bool inFinish );
 	std::ostream &TARGS( bool inFinish, int targState );
+	std::ostream &GOTO( NameInst *name, bool inFinish );
+	std::ostream &CALL( NameInst *name, int targState, bool inFinish );
+	std::ostream &RET( bool inFinish );
+	std::ostream &CALLE( InlineItem *ilItem, int targState, bool inFinish );
+	std::ostream &GOTOE( InlineItem *ilItem, bool inFinish );
+	std::ostream &NEXT( NameInst *name, bool inFinish );
+	std::ostream &NEXTE( InlineItem *ilItem, bool inFinish );
+
 
 protected:
 	/* Called from GotoCodeGen::STATE_GOTOS just before writing the gotos for
@@ -57,21 +62,12 @@ protected:
 /*
  * class CIpGotoCodeGen
  */
-class CIpGotoCodeGen : public IpGotoCodeGen
+class CIpGotoCodeGen : public IpGotoCodeGen, public CCodeGen
 {
 public:
 	CIpGotoCodeGen( char *fsmName, ParseData *parseData, 
 			RedFsmAp *redFsm, std::ostream &out );
 
-	std::ostream &GOTO( NameInst *name, bool inFinish );
-	std::ostream &CALL( NameInst *name, int targState, bool inFinish );
-	std::ostream &RET( bool inFinish );
-	std::ostream &CALLE( InlineItem *ilItem, int targState, bool inFinish );
-	std::ostream &GOTOE( InlineItem *ilItem, bool inFinish );
-	std::ostream &NEXT( NameInst *name, bool inFinish );
-	std::ostream &NEXTE( InlineItem *ilItem, bool inFinish );
-
-	virtual void writeOutHeader() { C_HEADER(); }
 	virtual void writeOutCode();
 };
 
@@ -79,21 +75,12 @@ public:
 /*
  * class CppIpGotoCodeGen
  */
-class CppIpGotoCodeGen : public IpGotoCodeGen
+class CppIpGotoCodeGen : public IpGotoCodeGen, public CppCodeGen
 {
 public:
 	CppIpGotoCodeGen( char *fsmName, ParseData *parseData, 
 			RedFsmAp *redFsm, std::ostream &out );
 
-	std::ostream &GOTO( NameInst *name, bool inFinish );
-	std::ostream &CALL( NameInst *name, int targState, bool inFinish );
-	std::ostream &RET( bool inFinish );
-	std::ostream &CALLE( InlineItem *ilItem, int targState, bool inFinish );
-	std::ostream &GOTOE( InlineItem *ilItem, bool inFinish );
-	std::ostream &NEXT( NameInst *name, bool inFinish );
-	std::ostream &NEXTE( InlineItem *ilItem, bool inFinish );
-
-	virtual void writeOutHeader() { CPP_HEADER(); }
 	virtual void writeOutCode();
 };
 
@@ -101,21 +88,12 @@ public:
 /*
  * class ObjCIpGotoCodeGen
  */
-class ObjCIpGotoCodeGen : public IpGotoCodeGen
+class ObjCIpGotoCodeGen : public IpGotoCodeGen, public ObjCCodeGen
 {
 public:
 	ObjCIpGotoCodeGen( char *fsmName, ParseData *parseData, 
 			RedFsmAp *redFsm, std::ostream &out );
 
-	std::ostream &GOTO( NameInst *name, bool inFinish );
-	std::ostream &CALL( NameInst *name, int targState, bool inFinish );
-	std::ostream &RET( bool inFinish );
-	std::ostream &CALLE( InlineItem *ilItem, int targState, bool inFinish );
-	std::ostream &GOTOE( InlineItem *ilItem, bool inFinish );
-	std::ostream &NEXT( NameInst *name, bool inFinish );
-	std::ostream &NEXTE( InlineItem *ilItem, bool inFinish );
-
-	virtual void writeOutHeader() { OBJC_HEADER(); }
 	virtual void writeOutCode();
 };
 
