@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2004 Adrian Thurston <adriant@ragel.ca>
+ *  Copyright 2001-2005 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -51,7 +51,6 @@ char *defExtension = ".c";
 /* Io globals. */
 ostream *outStream = 0;
 output_filter *outFilter = 0;
-char *inputFile = 0;
 char *outputFile = 0;
 
 /* Controls minimization. */
@@ -154,7 +153,7 @@ char *fileNameFromStem( char *stemFile, char *suffix )
 }
 
 /* Global parse data pointer. */
-extern ParseData *pd;
+extern InputData *id;
 
 /* Total error count. */
 int gblErrorCount = 0;
@@ -163,8 +162,8 @@ int gblErrorCount = 0;
 ostream &error()
 {
 	/* Keep the error count. */
-	if ( pd != 0 )
-		pd->errorCount += 1;
+	if ( id != 0 && id->pd != 0 )
+		id->pd->errorCount += 1;
 	gblErrorCount += 1;
 	
 	cerr << PROGNAME ": ";
@@ -175,11 +174,11 @@ ostream &error()
 ostream &error( const BISON_YYLTYPE &loc )
 {
 	/* Keep the error count. */
-	if ( pd != 0 )
-		pd->errorCount += 1;
+	if ( id->pd != 0 )
+		id->pd->errorCount += 1;
 	gblErrorCount += 1;
 
-	cerr << inputFile << ":" << loc.first_line << ":" << 
+	cerr << id->fileName << ":" << loc.first_line << ":" << 
 			loc.first_column << ": ";
 	return cerr;
 }
@@ -188,11 +187,11 @@ ostream &error( const BISON_YYLTYPE &loc )
 ostream &error( const InputLoc &loc )
 {
 	/* Keep the error count. */
-	if ( pd != 0 )
-		pd->errorCount += 1;
+	if ( id->pd != 0 )
+		id->pd->errorCount += 1;
 	gblErrorCount += 1;
 
-	cerr << inputFile << ":" << loc.line << ":" << 
+	cerr << id->fileName << ":" << loc.line << ":" << 
 			loc.col << ": ";
 	return cerr;
 }
@@ -200,25 +199,25 @@ ostream &error( const InputLoc &loc )
 ostream &error( int first_line, int first_column )
 {
 	/* Keep the error count. */
-	if ( pd != 0 )
-		pd->errorCount += 1;
+	if ( id->pd != 0 )
+		id->pd->errorCount += 1;
 	gblErrorCount += 1;
 
-	cerr << inputFile << ":" << first_line << ":" << first_column << ": ";
+	cerr << id->fileName << ":" << first_line << ":" << first_column << ": ";
 	return cerr;
 }
 
 /* Print the opening to a warning, then return the error ostream. */
 ostream &warning( )
 {
-	cerr << inputFile << ": warning: ";
+	cerr << id->fileName << ": warning: ";
 	return cerr;
 }
 
 /* Print the opening to a warning in the input, then return the error ostream. */
 ostream &warning( const InputLoc &loc )
 {
-	cerr << inputFile << ":" << loc.line << ":" << 
+	cerr << id->fileName << ":" << loc.line << ":" << 
 			loc.col << ": warning: ";
 	return cerr;
 }
@@ -226,7 +225,7 @@ ostream &warning( const InputLoc &loc )
 /* Print the opening to a warning in the input, then return the error ostream. */
 std::ostream &warning( int first_line, int first_column )
 {
-	cerr << inputFile << ":" << first_line << ":" << 
+	cerr << id->fileName << ":" << first_line << ":" << 
 			first_column << ": warning: ";
 	return cerr;
 }
@@ -262,8 +261,8 @@ void escapeLineDirectivePath( std::ostream &out, char *path )
 /* Main, process args and call yyparse to start scanning input. */
 int main(int argc, char **argv)
 {
-	/* Paramater specification. */
 	ParamCheck pc("o:enmabjkcCJVdS:M:T:F:G:vHh?-:", argc, argv);
+	char *inputFile = 0;
 
 	while ( pc.check() ) {
 		switch ( pc.state ) {
@@ -523,7 +522,9 @@ int main(int argc, char **argv)
 		outStream = &cout;
 	}
 
-	if ( outputFormat == OutCCode || outputFormat == OutCppCode || outputFormat == OutObjCCode ) {
+	if ( outputFormat == OutCCode || outputFormat == OutCppCode || 
+			outputFormat == OutObjCCode )
+	{
 		/* Put a header on the output to indicate that the file was machine generated. */
 		*outStream <<
 			"/* Automatically generated by Ragel from \"" << inputFile << "\".\n"
@@ -540,8 +541,8 @@ int main(int argc, char **argv)
 		*outStream << "\"\n";
 	}
 
-	/* Make a parse data for the first fsm machine. */
-	pd = new ParseData( inputFile );
+	/* Create the input data for the top level file. */
+	id = new InputData( inputFile, 0, 0 );
 
 	/* Parse the input! */
 	yyparse();
