@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2004 Adrian Thurston <adriant@ragel.ca>
+ *  Copyright 2001-2005 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -72,11 +72,6 @@ Action::Action( const InputLoc &loc, char *name,
 {
 }
 
-Action::~Action()
-{
-	delete[] name;
-}
-
 /* Perform minimization after an operation according 
  * to the command line args. */
 void afterOpMinimize( FsmAp *fsm )
@@ -594,27 +589,28 @@ bool NameInst::anyRefsRec()
 
 /* Initialize the structure that will collect info during the parse of a
  * machine. */
-ParseData::ParseData( const String &fileName )
+ParseData::ParseData( char *fileName, char *fsmName, 
+		const InputLoc &sectionLoc )
 :	
-	actionIndex(0),
-	baseClause(0),
 	nextPriorKey(0),
 	/* 0 is reserved for global error actions. */
 	nextLocalErrKey(1),
 	nextNameId(0),
-
-	machineGiven(false), 
 	alphType(AT_Char),
 	elementType(0),
 	getKeyExpr(0),
 	nextContextId(0),
+	lowerNum(0),
+	upperNum(0),
 	fileName(fileName),
+	fsmName(fsmName),
+	sectionLoc(sectionLoc),
 	errorCount(0),
-
 	curActionOrd(0),
 	curPriorOrd(0),
-
-	rootName(0)
+	rootName(0),
+	nextLongestMatchId(0),
+	codeGen(0)
 {
 	/* Initialize the dictionary of graphs. This is our symbol table. The
 	 * initialization needs to be done on construction which happens at the
@@ -626,13 +622,9 @@ ParseData::ParseData( const String &fileName )
 /* Clean up the data collected during a parse. */
 ParseData::~ParseData()
 {
-	if ( actionIndex != 0 )
-		delete[] actionIndex;
-
 	/* Delete all the nodes in the action list. Will cause all the
 	 * string data that represents the actions to be deallocated. */
 	actionList.empty();
-	dataList.empty();
 	initCodeList.empty();
 }
 
@@ -903,6 +895,9 @@ void ParseData::resolveNameRefs( InlineList *inlineList, Action *action )
 			default:
 				break;
 		}
+		/* Some of the item types may have children. */
+		if ( item->children != 0 )
+			resolveNameRefs( item->children, action );
 	}
 }
 
@@ -1000,7 +995,7 @@ void ParseData::initGraphDict( )
 }
 
 /* Set the alphabet type. If type types are not valid returns false. */
-bool ParseData::setAlphType( String s1, String s2 )
+bool ParseData::setAlphType( char *s1, char *s2 )
 {
 	bool valid = false;
 	if ( strcmp( s1, "unsigned" ) == 0 ) {
@@ -1021,7 +1016,7 @@ bool ParseData::setAlphType( String s1, String s2 )
 }
 
 /* Set the alphabet type. If type types are not valid returns false. */
-bool ParseData::setAlphType( String s1 )
+bool ParseData::setAlphType( char *s1 )
 {
 	bool valid = false;
 	if ( strcmp( s1, "char" ) == 0 ) {
@@ -1041,7 +1036,7 @@ bool ParseData::setAlphType( String s1 )
 
 void ParseData::setLowerUpperRange( )
 {
-	if ( lowerNum.length() != 0 ) {
+	if ( lowerNum != 0 ) {
 		/* If ranges are given then interpret the alphabet type. */
 		lowKey = makeFsmKeyNum( lowerNum, rangeLowLoc, this );
 		highKey = makeFsmKeyNum( upperNum, rangeHighLoc, this );
@@ -1087,23 +1082,6 @@ void ParseData::initKeyOps( )
 	keyOps.highKey = highKey;
 }
 
-/* Fill the action index for easy access to the actions. */
-void ParseData::fillActionIndex()
-{
-	/* Make the array of pointers to action list elements. */
-	actionIndex = new Action*[actionList.size()];
-
-	int nextActionId = 0;
-	for ( ActionList::Iter act = actionList; act.lte(); act++ ) {
-		/* Only ever interested in referenced actions. */
-		if ( act->actionRefs.length() > 0 ) {
-			act->actionId = nextActionId;
-			actionIndex[nextActionId] = act;
-			nextActionId += 1;
-		}
-	}
-}
-
 void ParseData::printNameInst( NameInst *nameInst, int level )
 {
 	for ( int i = 0; i < level; i++ )
@@ -1115,6 +1093,35 @@ void ParseData::printNameInst( NameInst *nameInst, int level )
 		printNameInst( *name, level+1 );
 }
 
+/* Remove duplicates of unique actions from an action table. */
+void ParseData::removeDups( ActionTable &table )
+{
+	/* Scan through the table looking for unique actions to 
+	 * remove duplicates of. */
+	for ( int i = 0; i < table.length(); i++ ) {
+		/* Remove any duplicates ahead of i. */
+		for ( int r = i+1; r < table.length(); ) {
+			if ( table[r].value == table[i].value )
+				table.ActionTableVect::remove(r);
+			else
+				r += 1;
+		}
+	}
+}
+
+/* Remove duplicates from action lists. This operates only on transition and
+ * eof action lists and so should be called once all actions have been
+ * transfered to their final resting place. */
+void ParseData::removeActionDups( FsmAp *graph )
+{
+	/* Loop all states. */
+	for ( StateList::Iter state = graph->stateList; state.lte(); state++ ) {
+		/* Loop all transitions. */
+		for ( OutIter outIt( state ); ! outIt.end(); outIt++ )
+			removeDups( outIt.trans->actionTable );
+		removeDups( state->eofActionTable );
+	}
+}
 
 /* Make the graph from a graph dict node. Does minimization and state sorting. */
 FsmAp *ParseData::makeInstance( GraphDictEl *gdNode )
@@ -1131,14 +1138,11 @@ FsmAp *ParseData::makeInstance( GraphDictEl *gdNode )
 	 * All state construction is now complete.
 	 */
 
-	/* Transfer actions from out action tables to transitions or to the eof
-	 * action table. */
-	for ( StateSet::Iter state = graph->finStateSet; state.lte(); state++ )
-		graph->transferOutActions( *state );
-
 	/* Transfer global error actions. */
 	for ( StateList::Iter state = graph->stateList; state.lte(); state++ )
 		graph->transferErrorActions( state, 0 );
+	
+	removeActionDups( graph );
 
 	/* Remove unreachable states. There should be no dead end states however.
 	 * The subtract and intersection operators are the only places where they
@@ -1199,9 +1203,6 @@ FsmAp *ParseData::makeSpecific( GraphDictEl *gdNode )
 	initNameWalk();
 	gdNode->value->resolveNameRefs( this );
 
-	/* Assign action ids and fill a list of pointers indexed by ids. */
-	fillActionIndex();
-
 	/* Just building the specified graph. */
 	initNameWalk();
 	FsmAp *mainGraph = makeInstance( gdNode );
@@ -1223,9 +1224,6 @@ FsmAp *ParseData::makeAll()
 	resolveInitNameRefs();
 	resolveActionNameRefs();
 
-	/* Assign action ids and fill a list of pointers indexed by ids. */
-	fillActionIndex();
-
 	FsmAp *mainGraph = 0;
 	FsmAp **graphs = new FsmAp*[instanceList.length()];
 	int numOthers = 0;
@@ -1289,18 +1287,14 @@ void ParseData::dumpFsm( GraphDictEl *dictEl )
 	dump.dumpGraph( );
 }
 
-/* Four different things can happen here, there can be an error, the machine
+/* Four different things can happen here, ?, there can be an error, the machine
  * can be ignore, the main machine can be output by default and a specific
  * machine can be output. */
 void ParseData::dumpFsm()
 {
-	/* Bail if there was no machine given. Nothing to output. */
-	if ( ! machineGiven )
-		return;
-
 	/* A machine was given, proceed. */
 	GraphDictEl *dictEl = 0;
-	if ( shouldGenerate( dictEl ) )
+	if ( shouldGenNonCode( dictEl ) )
 		dumpFsm( dictEl );
 }
 
@@ -1340,7 +1334,10 @@ void ParseData::generateGraphviz( GraphDictEl *dictEl )
 	dotGen.writeDotFile();
 }
 
-bool ParseData::shouldGenerate( GraphDictEl *&genDictEl )
+/* Determine if we are interested in this machine spec, and if so, are we
+ * interested in the whole spec or just one machine. This depends on the -M
+ * and -S options. */
+bool ParseData::shouldGenNonCode( GraphDictEl *&genDictEl )
 {
 	bool shouldGenerate = false;
 
@@ -1357,7 +1354,7 @@ bool ParseData::shouldGenerate( GraphDictEl *&genDictEl )
 			}
 			else {
 				/* No recovery action. Fsm is skipped. */
-				error(fsmStartSecLoc) << "machine " << machineName << 
+				error(sectionLoc) << "machine " << machineName << 
 							" not found in " << fsmName << endl;
 			}
 		}
@@ -1370,7 +1367,7 @@ bool ParseData::shouldGenerate( GraphDictEl *&genDictEl )
 			}
 			else {
 				/* No recovery action, fsm is skipped. */
-				error(fsmStartSecLoc) << "main graph not defined in \"" 
+				error(sectionLoc) << "main graph not defined in \"" 
 						<< fsmName << "\"" << endl;
 			} 
 		}
@@ -1378,17 +1375,22 @@ bool ParseData::shouldGenerate( GraphDictEl *&genDictEl )
 	return shouldGenerate;
 }
 
-void ParseData::generateGraphviz( )
+void ParseData::endCodeGen( const InputLoc &loc )
 {
-	/* If there was no machine given, we cannot generate graphviz output. */
-	if ( ! machineGiven )
+	/* If any errors have occured in the input file then don't write anything. */
+	if ( gblErrorCount > 0 )
 		return;
 
+	codeGen->endCodeGen( loc );
+}
+
+void ParseData::generateGraphviz( )
+{
 	/* A machine was given, proceed. In when generatign g*/
 	GraphDictEl *dictEl = 0;
 	if ( !graphvizDone ) {
 		/* For graphviz generate only the first graph. */
-		if ( shouldGenerate( dictEl ) ) {
+		if ( shouldGenNonCode( dictEl ) ) {
 			generateGraphviz( dictEl );
 			graphvizDone = true;
 		}
@@ -1397,82 +1399,81 @@ void ParseData::generateGraphviz( )
 
 
 /* Generate the codegen depending on the command line options given. */
-FsmCodeGen *ParseData::makeCodeGen( RedFsmAp *redFsm )
+void ParseData::makeCodeGen()
 {
-	FsmCodeGen *codeGen = 0;
 	switch ( outputFormat ) {
 	case OutCCode:
 		switch ( codeStyle ) {
 		case GenTables:
-			codeGen = new CTabCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new CTabCodeGen( fsmName, this, 0, *outStream );
 			break;
 		case GenFTables:
-			codeGen = new CFTabCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new CFTabCodeGen( fsmName, this, 0, *outStream );
 			break;
 		case GenFlat:
-			codeGen = new CFlatCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new CFlatCodeGen( fsmName, this, 0, *outStream );
 			break;
 		case GenFFlat:
-			codeGen = new CFFlatCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new CFFlatCodeGen( fsmName, this, 0, *outStream );
 			break;
 		case GenGoto:
-			codeGen = new CGotoCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new CGotoCodeGen( fsmName, this, 0, *outStream );
 			break;
 		case GenFGoto:
-			codeGen = new CFGotoCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new CFGotoCodeGen( fsmName, this, 0, *outStream );
 			break;
 		case GenIpGoto:
-			codeGen = new CIpGotoCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new CIpGotoCodeGen( fsmName, this, 0, *outStream );
 			break;
 		}
 		break;
 	case OutCppCode:
 		switch ( codeStyle ) {
 		case GenTables:
-			codeGen = new CppTabCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new CppTabCodeGen( fsmName, this, 0, *outStream );
 			break;
 		case GenFTables:
-			codeGen = new CppFTabCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new CppFTabCodeGen( fsmName, this, 0, *outStream );
 			break;
 		case GenFlat:
-			codeGen = new CppFlatCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new CppFlatCodeGen( fsmName, this, 0, *outStream );
 			break;
 		case GenFFlat:
-			codeGen = new CppFFlatCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new CppFFlatCodeGen( fsmName, this, 0, *outStream );
 			break;
 		case GenGoto:
-			codeGen = new CppGotoCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new CppGotoCodeGen( fsmName, this, 0, *outStream );
 			break;
 		case GenFGoto:
-			codeGen = new CppFGotoCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new CppFGotoCodeGen( fsmName, this, 0, *outStream );
 			break;
 		case GenIpGoto:
-			codeGen = new CppIpGotoCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new CppIpGotoCodeGen( fsmName, this, 0, *outStream );
 			break;
 		}
 		break;
 	case OutObjCCode:
 		switch ( codeStyle ) {
 		case GenTables:
-			codeGen = new ObjCTabCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new ObjCTabCodeGen( fsmName, this, 0, *outStream );
 			break;
 		case GenFTables:
-			codeGen = new ObjCFTabCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new ObjCFTabCodeGen( fsmName, this, 0, *outStream );
 			break;
 		case GenFlat:
-			codeGen = new ObjCFlatCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new ObjCFlatCodeGen( fsmName, this, 0, *outStream );
 			break;
 		case GenFFlat:
-			codeGen = new ObjCFFlatCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new ObjCFFlatCodeGen( fsmName, this, 0, *outStream );
 			break;
 		case GenGoto:
-			codeGen = new ObjCGotoCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new ObjCGotoCodeGen( fsmName, this, 0, *outStream );
 			break;
 		case GenFGoto:
-			codeGen = new ObjCFGotoCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new ObjCFGotoCodeGen( fsmName, this, 0, *outStream );
 			break;
 		case GenIpGoto:
-			codeGen = new ObjCIpGotoCodeGen( fsmName, this, redFsm, *outStream );
+			codeGen = new ObjCIpGotoCodeGen( fsmName, this, 0, *outStream );
 			break;
 		}
 		break;
@@ -1482,102 +1483,151 @@ FsmCodeGen *ParseData::makeCodeGen( RedFsmAp *redFsm )
 	default:
 		assert( false );
 	}
-	return codeGen;
+}
+
+/* Check actions for bad uses of fsm directives */ 
+void ParseData::checkInlineList( Action *act, InlineList *inlineList )
+{
+	for ( InlineList::Iter item = *inlineList; item.lte(); item++ ) {
+		/* Transition action checks. */
+		if ( act->numTransRefs > 0 ) {
+			switch ( item->type ) {
+			default:
+				break;
+			}
+		}
+
+		/* EOF checks. */
+		if ( act->numEofRefs > 0 ) {
+			switch ( item->type ) {
+			case InlineItem::PChar: 
+				warning(item->loc) << "pointer to current element is always null in "
+						"EOF action code" << endl;
+				break;
+			case InlineItem::Char: 
+				warning(item->loc) << "reference to current character in "
+						"EOF action will result in null dereference" << endl;
+				break;
+			case InlineItem::Exec:
+				warning(item->loc) << "restarting with a new buffer has no effect in "
+						"EOF action code" << endl;
+				break;
+			case InlineItem::Buf:
+				warning(item->loc) << "buffer data is undefined "
+						"EOF action code" << endl;
+				break;
+			case InlineItem::BufLen:
+				warning(item->loc) << "buffer length is undefined "
+						"EOF action code" << endl;
+				break;
+			default:
+				break;
+			}
+		}
+
+		/* Recurse. */
+		if ( item->children != 0 )
+			checkInlineList( act, item->children );
+	}
+}
+
+/* Checks for bad usage of directives in action code. */
+void ParseData::checkActionCodeContents()
+{
+	/* Check if there are any calls in action code. */
+	for ( ActionList::Iter act = actionList; act.lte(); act++ )
+		checkInlineList( act, act->inlineList );
 }
 
 /* Generate the code for an fsm. Assumes parseData is set up properly. Called
  * by parser code. */
 void ParseData::generateCode( )
 {
-	/* The machine that will take the compressed fsm and the code generator. */
-	RedFsmAp *redFsm = 0;
-	FsmCodeGen *codeGen = 0;
-
-	/* Nothing to do if there is no datalist or machine given. */
-	if ( dataList.length() == 0 && ! machineGiven )
+	/* Look for the main instance. The parser prevents main from going
+	 * into the graph dictionary as a variable def and not an instance. */
+	GraphDictEl *mainEl = graphDict.find( machineMain );
+	if ( mainEl == 0 ) {
+		/* No recovery action, fsm is skipped. */
+		error(sectionLoc) << "main graph not defined in \"" 
+				<< fsmName << "\"" << endl;
 		return;
-	
-	/* If the graph dict is given, find the main machine. */
-	if ( machineGiven ) {
-		/* Look for the main instance. The parser prevents main from going
-		 * into the graph dictionary as a variable def and not an instance. */
-		GraphDictEl *mainEl = graphDict.find( machineMain );
-		if ( mainEl == 0 ) {
-			/* No recovery action, fsm is skipped. */
-			error(fsmStartSecLoc) << "main graph not defined in \"" 
-					<< fsmName << "\"" << endl;
-			return;
-		}
-
-		/* Interpret the alphabet type. */
-		setLowerUpperRange();
-		initKeyOps();
-
-		/* Make the main graph. */
-		FsmAp *mainGraph = makeGraph( 0 );
-
-		/* Make the machine. */
-		redFsm = new RedFsmAp( mainGraph, true );
-
-		/* Order the states. */
-		redFsm->depthFirstOrdering();
-
-		if ( codeStyle == GenGoto || codeStyle == GenFGoto || codeStyle == GenIpGoto ) {
-			/* For goto driven machines we can keep the original depth
-			 * first ordering because it's ok if the state ids are not
-			 * sequential. Split the the ids by final state status. */
-			redFsm->sortStateIdsByFinal();
-		}
-		else {
-			/* For table driven machines the location of the state is used to
-			 * identify it so the states must be sorted by their final ids.
-			 * Though having a deterministic ordering is important,
-			 * specifically preserving the depth first ordering is not because
-			 * states are stored in tables. */
-			redFsm->sortStatesByFinal();
-			redFsm->sequentialStateIds();
-		}
-
-		/* Find the first final state. This is the final state with the lowest
-		 * id.  */
-		redFsm->findFirstFinState();
-
-		/* Choose default transitions and the single transition. */
-		redFsm->chooseDefaultSpan();
-		
-		/* Maybe do flat expand, otherwise choose single. */
-		if ( codeStyle == GenFlat || codeStyle == GenFFlat )
-			redFsm->makeFlat();
-		else
-			redFsm->chooseSingle();
 	}
 
+	/* Interpret the alphabet type. */
+	setLowerUpperRange();
+	initKeyOps();
+
+	/* Make the main graph. */
+	FsmAp *mainGraph = makeGraph( 0 );
+
+	/* Make the machine. */
+	RedFsmAp *redFsm = new RedFsmAp( mainGraph, true );
+
+	/* Order the states. */
+	redFsm->depthFirstOrdering();
+
+	if ( codeStyle == GenGoto || codeStyle == GenFGoto || codeStyle == GenIpGoto ) {
+		/* For goto driven machines we can keep the original depth
+		 * first ordering because it's ok if the state ids are not
+		 * sequential. Split the the ids by final state status. */
+		redFsm->sortStateIdsByFinal();
+	}
+	else {
+		/* For table driven machines the location of the state is used to
+		 * identify it so the states must be sorted by their final ids.
+		 * Though having a deterministic ordering is important,
+		 * specifically preserving the depth first ordering is not because
+		 * states are stored in tables. */
+		redFsm->sortStatesByFinal();
+		redFsm->sequentialStateIds();
+	}
+
+	/* Find the first final state. This is the final state with the lowest
+	 * id.  */
+	redFsm->findFirstFinState();
+
+	/* Choose default transitions and the single transition. */
+	redFsm->chooseDefaultSpan();
+		
+	/* Maybe do flat expand, otherwise choose single. */
+	if ( codeStyle == GenFlat || codeStyle == GenFFlat )
+		redFsm->makeFlat();
+	else
+		redFsm->chooseSingle();
+
 	/* If any errors have occured in the input file then don't write anything. */
 	if ( gblErrorCount > 0 )
 		return;
 
 	/* Make a code generator that will output the header/code. */
-	codeGen = makeCodeGen( redFsm );
+	if ( codeGen == 0 )
+		makeCodeGen();
+	codeGen->redFsm = redFsm;
 
 	/* Write line directive. */
 	codeGen->startCodeGen();
 
-	/* If the struct element was used then output the header portion. */
-	if ( dataList.length() > 0 )
-		codeGen->writeOutHeader( );
-	
-	/* If there was a machine given, gather info on it and write it out. */
-	if ( machineGiven ) {
-		codeGen->analyzeMachine();
-		codeGen->writeOutCode( );
-	}
+	/* Anlayze Machine will find the final action reference counts, among
+	 * other things. We will use these in reporting the usage
+	 * of fsm directives in action code. */
+	codeGen->analyzeMachine();
+	checkActionCodeContents();
+	codeGen->writeOutCode( );
 
-	/* Write line directive. */
-	codeGen->endCodeGen();
-
-	/* Done with the code generator. */
-	delete codeGen;
-	if ( redFsm != 0 )
-		delete redFsm;
+	/* Done with generated machine. */
+	delete redFsm;
 }
 
+void ParseData::generateInterface( )
+{
+	/* If any errors have occured in the input file then don't write anything. */
+	if ( gblErrorCount > 0 )
+		return;
+
+	/* Make a code generator that will output the header/code. */
+	if ( codeGen == 0 )
+		makeCodeGen();
+
+	codeGen->startCodeGen();
+	codeGen->writeOutInterface( );
+}
