@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2004 Adrian Thurston <adriant@ragel.ca>
+ *  Copyright 2001-2005 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -24,7 +24,6 @@
 
 #include <limits.h>
 #include "ptreetypes.h"
-#include "astring.h"
 #include "avlmap.h"
 #include "bstmap.h"
 #include "astring.h"
@@ -32,6 +31,7 @@
 #include "dlist.h"
 #include "fsmgraph.h"
 #include "compare.h"
+#include "vector.h"
 
 /* Forwards. */
 struct RedFsmAp;
@@ -87,6 +87,8 @@ struct InputLoc
 {
 	InputLoc( ) 
 		: line(0), col(0) { }
+	InputLoc( int line, int col ) 
+		: line(line), col(col) { }
 	InputLoc( const BISON_YYLTYPE &loc );
 
 	int line;
@@ -127,7 +129,6 @@ struct Action
 public:
 	Action( const InputLoc &loc, char *name, InlineList *inlineList, 
 			const NameRefList &nameRefs );
-	~Action();
 
 	/* Key for action dictionary. */
 	char *getKey() const { return name; }
@@ -333,7 +334,7 @@ struct ParseData
 {
 	/* Create a new parse data object. This is done at the beginning of every
 	 * fsm specification. */
-	ParseData( const String &fileName );
+	ParseData( char *fileName, char *fsmName, const InputLoc &sectionLoc );
 	~ParseData();
 
 	/*
@@ -364,12 +365,13 @@ struct ParseData
 	void resolveActionNameRefs();
 	void resolveInitNameRefs();
 
-	/* Make an index of pointers to the action blocks. */
-	void fillActionIndex();
-
 	/* Set the alphabet type. If type types are not valid returns false. */
-	bool setAlphType( String s1, String s2 );
-	bool setAlphType( String s1 );
+	bool setAlphType( char *s1, char *s2 );
+	bool setAlphType( char *s1 );
+
+	/* Unique actions. */
+	void removeDups( ActionTable &actionTable );
+	void removeActionDups( FsmAp *graph );
 
 	/* Dumping the name instantiation tree. */
 	void printNameInst( NameInst *nameInst, int level );
@@ -381,7 +383,8 @@ struct ParseData
 	FsmAp *makeGraph( GraphDictEl *gdNode );
 
 	/* Resolving machine paths. */
-	bool shouldGenerate( GraphDictEl *&genDictEl );
+	bool shouldGenNonCode( GraphDictEl *&genDictEl );
+	void endCodeGen( const InputLoc &loc );
 
 	/* Dumping and printing. */
 	void dumpFsm( GraphDictEl *gdNode );
@@ -392,10 +395,15 @@ struct ParseData
 	void generateGraphviz( GraphDictEl *gdNode );
 	void generateGraphviz();
 
+	/* Checking the contents of actions. */
+	void ParseData::checkInlineList( Action *act, InlineList *inlineList );
+	void ParseData::checkActionCodeContents();
+
 	/* Generate and write out the fsm. */
 	void generateCode( );
+	void generateInterface( );
 	void doGenerateCode( GraphDictEl *gdNode );
-	FsmCodeGen *makeCodeGen( RedFsmAp *machine2 );
+	void makeCodeGen();
 
 	/* Set the lower and upper range from the lower and upper number keys. */
 	void setLowerUpperRange( );
@@ -429,11 +437,6 @@ struct ParseData
 
 	/* List of actions. Will be pasted into a switch statement. */
 	ActionList actionList;
-	Action **actionIndex;
-
-	/* List of sections of data to add to the fsm struct. */
-	InlineBlockList dataList;
-	char *baseClause;
 
 	/* The id of the next priority name and label. */
 	int nextPriorKey, nextLocalErrKey, nextNameId;
@@ -444,9 +447,6 @@ struct ParseData
 
 	int curDefLocalErrKey;
 
-	/* If a machine is given this is set. */
-	bool machineGiven;
-
 	/* Code sections put in the fsms init routine. */
 	InlineBlockList initCodeList;
 
@@ -464,20 +464,17 @@ struct ParseData
 	int nextContextId;
 
 	/* The alphabet range. */
-	String lowerNum, upperNum;
+	char *lowerNum, *upperNum;
 	long lowKey, highKey;
 	InputLoc rangeLowLoc, rangeHighLoc;
 
 	/* Key operators. */
 	KeyOps keyOps;
 
-	/* The name and location of the fsm. */
-	String fsmName;
-	InputLoc fsmStartSecLoc;
-	InputLoc fsmEndSecLoc;
-
-	/* The name of the file the fsm is from. */
-	String fileName;
+	/* The name of the file the fsm is from, and the spec name. */
+	char *fileName;
+	char *fsmName;
+	InputLoc sectionLoc;
 
 	/* Number of errors encountered parsing the fsm spec. */
 	int errorCount;
@@ -502,6 +499,12 @@ struct ParseData
 
 	/* Make name ids to name inst pointers. */
 	NameInst **nameIndex;
+
+	/* Counter for assigning ids to longest match items. */
+	int nextLongestMatchId;
+
+	/* The code generator, subclasses exist for output style/language. */
+	FsmCodeGen *codeGen;
 };
 
 void afterOpMinimize( FsmAp *fsm );
@@ -515,4 +518,55 @@ FsmAp *makeBuiltin( BuiltinMachine builtin, ParseData *pd );
 
 void errorStateLabels( const NameSet &locations );
 
+/* Data used by the parser specific to the current file. Supports the include
+ * system, since a new parser is executed for each included file. */
+struct InputData
+{
+	InputData( char *fileName, char *includeSpec, char *includeTo ) :
+		pd(0), sectionName(0), defaultParseData(0), 
+		first_line(1), first_column(1), 
+		last_line(1), last_column(0), 
+		fileName(fileName), includeSpec(includeSpec), 
+		includeTo(includeTo), active(true), 
+		lookedUpSection(false) {}
+
+	/* For collecting a name references. */
+	NameRef nameRef;
+	NameRefList nameRefList;
+
+	/* The parse data. For each fsm spec, the parser collects things that it parses
+	 * in data structures in here. */
+	ParseData *pd;
+
+	/* The the source of the end of the section location varies, sometimes it's
+	 * the last token in the only statement or it is the closing '}' */
+	InputLoc sectionEndLoc;
+
+	char *sectionName;
+	ParseData *defaultParseData;
+
+	typedef Vector<char *> StructStack;
+	StructStack structStack;
+
+	int first_line;
+	int first_column;
+	int last_line;
+	int last_column;
+
+	char *fileName;
+
+	/* If this is an included file, this contains the specification to search
+	 * for. IncludeTo will contain the spec name that does the includng. */
+	char *includeSpec;
+	char *includeTo;
+
+	bool active;
+	bool lookedUpSection;
+
+	bool sawInterface;
+	bool sawMain;
+
+	InputLoc sectionLoc;
+};
+
 #endif /* _PARSEDATA_H */
