@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2004 Adrian Thurston <adriant@ragel.ca>
+ *  Copyright 2001-2004 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -108,6 +108,287 @@ void VarDef::resolveNameRefs( ParseData *pd )
 	pd->popNameScope( nameFrame );
 }
 
+void LongestMatch::makeActions( ParseData *pd )
+{
+	/* Make actions that set the action id. */
+	for ( LongestMatchList::Iter lmi = *longestMatchList; lmi.lte(); lmi++ ) {
+		/* For each part create actions for setting the match type.  We need
+		 * to do this so that the actions will go into the actionIndex. */
+		InlineList *inlineList = new InlineList;
+		char *actData = new char[50];
+		sprintf( actData, "act=%i;", lmi->longestMatchId );
+		inlineList->append( new InlineItem( lmi->action->loc, actData, InlineItem::Text ) );
+		char *actName = new char[50];
+		sprintf( actName, "store%i", lmi->longestMatchId );
+		Action *action = new Action( lmi->action->loc, actName, inlineList, NameRefList() );
+		action->actionRefs.append( pd->curNameInst );
+		pd->actionList.append( action );
+		lmi->setActId = action;
+	}
+
+	/* Make actions that execute the user action and restart on the last character. */
+	for ( LongestMatchList::Iter lmi = *longestMatchList; lmi.lte(); lmi++ ) {
+		/* For each part create actions for setting the match type.  We need
+		 * to do this so that the actions will go into the actionIndex. */
+		InlineList *inlineList = new InlineList;
+		inlineList->append( new InlineItem( lmi->action->loc, "tokend = _p; ", InlineItem::Text ) );
+		inlineList->append( new InlineItem( lmi->action->loc, lmi, InlineItem::LmAct ) );
+		inlineList->append( new InlineItem( lmi->action->loc, "\n\ttokstart = _p+1; ", InlineItem::Text ) );
+		NameRef nameRef;
+		nameRef.append("main");
+		inlineList->append( new InlineItem( InputLoc(), new NameRef(nameRef), InlineItem::Goto ) );
+		char *actName = new char[50];
+		sprintf( actName, "imm%i", lmi->longestMatchId );
+		Action *action = new Action( lmi->action->loc, actName, inlineList, NameRefList() );
+		action->actionRefs.append( pd->curNameInst );
+		pd->actionList.append( action );
+		lmi->actRestartImm = action;
+	}
+
+	/* Make actions that execute the user action and restart on the last character. */
+	for ( LongestMatchList::Iter lmi = *longestMatchList; lmi.lte(); lmi++ ) {
+		/* For each part create actions for setting the match type.  We need
+		 * to do this so that the actions will go into the actionIndex. */
+		InlineList *inlineList = new InlineList;
+		inlineList->append( new InlineItem( lmi->action->loc, lmi, InlineItem::LmAct ) );
+		static char cd3[] = "\n\ttokstart = tokend+1; _p = tokend; ";
+		inlineList->append( new InlineItem( InputLoc(), cd3, InlineItem::Text ) );
+		NameRef nameRef;
+		nameRef.append("main");
+		inlineList->append( new InlineItem( InputLoc(), new NameRef(nameRef), InlineItem::Goto ) );
+		char *actName = new char[50];
+		sprintf( actName, "lag%i", lmi->longestMatchId );
+		Action *action = new Action( lmi->action->loc, actName, inlineList, NameRefList() );
+		action->actionRefs.append( pd->curNameInst );
+		pd->actionList.append( action );
+		lmi->thisErrAction = action;
+	}
+
+	/* Make actions that execute the user action and restart on the last character
+	 * and that don't need tokend to be set (it is one back). */
+	for ( LongestMatchList::Iter lmi = *longestMatchList; lmi.lte(); lmi++ ) {
+		/* For each part create actions for setting the match type.  We need
+		 * to do this so that the actions will go into the actionIndex. */
+		InlineList *inlineList = new InlineList;
+		inlineList->append( new InlineItem( InputLoc(), "tokend = _p-1; " , InlineItem::Text ) );
+		inlineList->append( new InlineItem( lmi->action->loc, lmi, InlineItem::LmAct ) );
+		inlineList->append( new InlineItem( InputLoc(), "\n\ttokstart = _p--; ", InlineItem::Text ) );
+		NameRef nameRef;
+		nameRef.append("main");
+		inlineList->append( new InlineItem( InputLoc(), new NameRef(nameRef), InlineItem::Goto ) );
+		char *actName = new char[50];
+		sprintf( actName, "lagh%i", lmi->longestMatchId );
+		Action *action = new Action( lmi->action->loc, actName, inlineList, NameRefList() );
+		action->actionRefs.append( pd->curNameInst );
+		pd->actionList.append( action );
+		lmi->thisErrActionNoTokEnd = action;
+	}
+
+	/* Create the tokend action. */
+	InlineList *il2 = new InlineList;
+	il2->append( new InlineItem( InputLoc(), "tokend=", InlineItem::Text ) );
+	il2->append( new InlineItem( InputLoc(), (char*)0, InlineItem::PChar ) );
+	il2->append( new InlineItem( InputLoc(), ";", InlineItem::Text ) );
+	tokEndAction = new Action( InputLoc(1,1), "tokend", il2, NameRefList() );
+	tokEndAction->actionRefs.append( pd->curNameInst );
+	pd->actionList.append( tokEndAction );
+
+	/* Create the error action. */
+	InlineList *il3 = new InlineList;
+	il3->append( new InlineItem( InputLoc(), this, InlineItem::LmSwitch ) );
+	static char cd3[] = "\n\ttokstart = tokend+1; _p = tokend; ";
+	il3->append( new InlineItem( InputLoc(), cd3, InlineItem::Text ) );
+	NameRef nameRef;
+	nameRef.append("main");
+	il3->append( new InlineItem( InputLoc(), new NameRef(nameRef), InlineItem::Goto ) );
+	lmActSelect = new Action( InputLoc(1,1), "lagsel", il3, NameRefList() );
+	lmActSelect->actionRefs.append( pd->curNameInst );
+	pd->actionList.append( lmActSelect );
+}
+
+void LongestMatch::makeNameTree( ParseData *pd )
+{
+	/* Create an anonymous scope for the longest match. */
+	NameInst *prevNameInst = pd->curNameInst;
+	pd->curNameInst = pd->addNameInst( loc, 0, false );
+
+	/* Recurse into all parts of the longest match operator. */
+	for ( LongestMatchList::Iter lmi = *longestMatchList; lmi.lte(); lmi++ )
+		lmi->join->makeNameTree( pd );
+
+	/* Also make the longest match's actions at this point. */
+	makeActions( pd );
+
+	/* The name scope ends, pop the name instantiation. */
+	pd->curNameInst = prevNameInst;
+}
+
+void LongestMatch::resolveNameRefs( ParseData *pd )
+{
+	/* The longest match gets its own name scope. */
+	NameFrame nameFrame = pd->enterNameScope( true, 1 );
+
+	/* Take an action reference for each longest match item and recurse. */
+	for ( LongestMatchList::Iter lmi = *longestMatchList; lmi.lte(); lmi++ ) {
+		/* Record the reference if the item has an action. */
+		if ( lmi->action != 0 )
+			lmi->action->actionRefs.append( pd->localNameScope );
+
+		/* Recurse down the join. */
+		lmi->join->resolveNameRefs( pd );
+	}
+
+	/* The name scope ends, pop the name instantiation. */
+	pd->popNameScope( nameFrame );
+}
+
+void LongestMatch::runLonestMatch( ParseData *pd, FsmAp *graph, int tokEndActionOrd )
+{
+	/* Transfer the first item of non-empty lmAction tables to the item sets
+	 * of the states that follow. Exclude states that have no transitions out.
+	 * This must happen on a separate pass so that on each iteration of the
+	 * next pass we have the item set entries from all lmAction tables. */
+	for ( StateList::Iter st = graph->stateList; st.lte(); st++ ) {
+		for ( OutIter outIt( st ); outIt.lte(); outIt++ ) {
+			if ( outIt.trans->lmActionTable.length() > 0 ) {
+				LmActionTableEl *lmAct = outIt.trans->lmActionTable.data;
+				StateAp *toState = outIt.trans->toState;
+				assert( toState );
+
+				/* Check if there are transitions out, this may be a very
+				 * close approximation? Out transitions going nowhere?
+				 * FIXME: Check. */
+				if ( toState->outList.length() > 0 || toState->outDefault != 0 ) {
+					/* Fill the item sets. */
+					graph->markReachableFromHereStopFinal( toState );
+					for ( StateList::Iter ms = graph->stateList; ms.lte(); ms++ ) {
+						if ( ms->stateBits & SB_ISMARKED ) {
+							ms->lmItemSet.insert( lmAct->value );
+							ms->stateBits &= ~ SB_ISMARKED;
+						}
+					}
+				}
+			}
+		}
+	}
+
+	/* Set actions that do immediate token recognition, set the longest match part
+	 * id and set the token ending. */
+	for ( StateList::Iter st = graph->stateList; st.lte(); st++ ) {
+		for ( OutIter outIt( st ); outIt.lte(); outIt++ ) {
+			if ( outIt.trans->lmActionTable.length() > 0 ) {
+				LmActionTableEl *lmAct = outIt.trans->lmActionTable.data;
+				StateAp *toState = outIt.trans->toState;
+				assert( toState );
+
+				/* Check if there are transitions out, this may be a very
+				 * close approximation? Out transitions going nowhere?
+				 * FIXME: Check. */
+				if ( toState->outList.length() == 0 && toState->outDefault == 0 ) {
+					/* Can execute the immediate action for the longest match
+					 * part. */
+					outIt.trans->actionTable.setAction( lmAct->key, 
+							lmAct->value->actRestartImm );
+				}
+				else {
+					/* Look for non final states that have a non-empty item
+					 * set. If these are present then we need to record the
+					 * end of the token.  Also Find the highest item set
+					 * length reachable from here (excluding at transtions to
+					 * final states). */
+					bool nonFinalNonEmptyItemSet = false;
+					int count = 0;
+					graph->markReachableFromHereStopFinal( toState );
+					for ( StateList::Iter ms = graph->stateList; ms.lte(); ms++ ) {
+						if ( ms->stateBits & SB_ISMARKED ) {
+							if ( ms->lmItemSet.length() > 0 && !ms->isFinState() )
+								nonFinalNonEmptyItemSet = true;
+							if ( ms->lmItemSet.length() > count )
+								count = ms->lmItemSet.length();
+							ms->stateBits &= ~ SB_ISMARKED;
+						}
+					}
+
+					/* If there are reachable states that are not final and
+					 * have non empty item sets or that have an item set
+					 * length greater than one then we need to set tokend
+					 * because the error action that matches the token will
+					 * require it. */
+					if ( nonFinalNonEmptyItemSet || count > 1 )
+						outIt.trans->actionTable.setAction( tokEndActionOrd, tokEndAction );
+
+					/* Some states may not know which longest match item to
+					 * execute, must set it. */
+					if ( count > 1 ) {
+						/* There are transitions out, another match may come. */
+						outIt.trans->actionTable.setAction( lmAct->key, 
+								lmAct->value->setActId );
+					}
+				}
+			}
+		}
+	}
+
+	int lmErrActionOrd = pd->curActionOrd++;
+
+	/* Embed the error for recognizing a char. */
+	for ( StateList::Iter st = graph->stateList; st.lte(); st++ ) {
+		if ( st->lmItemSet.length() == 1 ) {
+			if ( st->isFinState() ) {
+				/* the error action out of final states don't need to have
+				 * tokend because the end of the token is always one back. */
+				st->errActionTable.setAction( lmErrActionOrd, 
+						st->lmItemSet[0]->thisErrActionNoTokEnd, 0 );
+			}
+			else {
+				st->errActionTable.setAction( lmErrActionOrd, 
+						st->lmItemSet[0]->thisErrAction, 0 );
+			}
+		}
+		else if ( st->lmItemSet.length() > 1 ) {
+			/* Need to use the select. Take note of the which items the select
+			 * is needed for so only the necessary actions are included. */
+			for ( LmItemSet::Iter plmi = st->lmItemSet; plmi.lte(); plmi++ )
+				(*plmi)->inLmSelect = true;
+			st->errActionTable.setAction( lmErrActionOrd, lmActSelect, 0 );
+		}
+	}
+}
+
+FsmAp *LongestMatch::walk( ParseData *pd )
+{
+	/* The longest match has it's own name scope. */
+	NameFrame nameFrame = pd->enterNameScope( true, 1 );
+
+	int tokEndActionOrd = pd->curActionOrd++;
+
+	/* Make each part of the longest match. */
+	FsmAp **parts = new FsmAp*[longestMatchList->length()];
+	LongestMatchList::Iter lmi = *longestMatchList; 
+	for ( int i = 0; lmi.lte(); lmi++, i++ ) {
+		/* Create the machine and embed the setting of the longest match id. */
+		parts[i] = lmi->join->walk( pd );
+		parts[i]->longMatchAction( pd->curActionOrd++, lmi );
+	}
+
+	/* Union machines one and up with machine zero. The grammar dictates that
+	 * there will always be at least one part. */
+	FsmAp *rtnVal = parts[0];
+	for ( int i = 1; i < longestMatchList->length(); i++ ) {
+		rtnVal->unionOp( parts[i] );
+		afterOpMinimize( rtnVal );
+	}
+
+	runLonestMatch( pd, rtnVal, tokEndActionOrd );
+
+	/* Pop the name scope. */
+	pd->popNameScope( nameFrame );
+
+	delete[] parts;
+	return rtnVal;
+}
+
+
 /* Construct with a location and the first expression. */
 Join::Join( const InputLoc &loc, Expression *expr )
 :
@@ -425,77 +706,103 @@ void FactorWithAug::assignActions( ParseData *pd, FsmAp *graph, int *actionOrd )
 	/* Assign actions. */
 	for ( int i = 0; i < actions.length(); i++ )  {
 		switch ( actions[i].type ) {
+		/* Transition actions. */
 		case at_start:
-			graph->startFsmAction( actionOrd[i], actions[i].action->actionId );
+			graph->startFsmAction( actionOrd[i], actions[i].action );
 			afterOpMinimize( graph );
 			break;
 		case at_all:
-			graph->allTransAction( actionOrd[i], actions[i].action->actionId );
+			graph->allTransAction( actionOrd[i], actions[i].action );
 			break;
 		case at_finish:
-			graph->finishFsmAction( actionOrd[i], actions[i].action->actionId );
+			graph->finishFsmAction( actionOrd[i], actions[i].action );
 			break;
 		case at_leave:
-			graph->leaveFsmAction( actionOrd[i], actions[i].action->actionId );
+			graph->leaveFsmAction( actionOrd[i], actions[i].action );
 			break;
 
+		/* Global error actions. */
 		case at_start_gbl_error:
-			graph->startErrorAction( actionOrd[i], actions[i].action->actionId, 0 );
+			graph->startErrorAction( actionOrd[i], actions[i].action, 0 );
 			afterOpMinimize( graph );
 			break;
 		case at_all_gbl_error:
-			graph->allErrorAction( actionOrd[i], actions[i].action->actionId, 0 );
+			graph->allErrorAction( actionOrd[i], actions[i].action, 0 );
 			break;
 		case at_finish_gbl_error:
-			graph->finishErrorAction( actionOrd[i], actions[i].action->actionId, 0 );
+			graph->finishErrorAction( actionOrd[i], actions[i].action, 0 );
 			break;
 		case at_leave_gbl_error:
-			graph->leaveErrorAction( actionOrd[i], actions[i].action->actionId, 0 );
+			graph->leaveErrorAction( actionOrd[i], actions[i].action, 0 );
 			break;
 		case at_start_finish_gbl_error:
-			graph->startErrorAction( actionOrd[i], actions[i].action->actionId, 0 );
-			graph->finishErrorAction( actionOrd[i], actions[i].action->actionId, 0 );
+			graph->startErrorAction( actionOrd[i], actions[i].action, 0 );
+			graph->finishErrorAction( actionOrd[i], actions[i].action, 0 );
 			afterOpMinimize( graph );
 			break;
 		case at_finish_leave_gbl_error:
-			graph->finishErrorAction( actionOrd[i], actions[i].action->actionId, 0 );
-			graph->leaveErrorAction( actionOrd[i], actions[i].action->actionId, 0 );
+			graph->finishErrorAction( actionOrd[i], actions[i].action, 0 );
+			graph->leaveErrorAction( actionOrd[i], actions[i].action, 0 );
 			break;
 
+		/* Local error actions. */
 		case at_start_local_error:
-			graph->startErrorAction( actionOrd[i], actions[i].action->actionId,
+			graph->startErrorAction( actionOrd[i], actions[i].action,
 					actions[i].localErrKey );
 			afterOpMinimize( graph );
 			break;
 		case at_all_local_error:
-			graph->allErrorAction( actionOrd[i], actions[i].action->actionId,
+			graph->allErrorAction( actionOrd[i], actions[i].action,
 					actions[i].localErrKey );
 			break;
 		case at_finish_local_error:
-			graph->finishErrorAction( actionOrd[i], actions[i].action->actionId,
+			graph->finishErrorAction( actionOrd[i], actions[i].action,
 					actions[i].localErrKey );
 			break;
 		case at_leave_local_error:
-			graph->leaveErrorAction( actionOrd[i], actions[i].action->actionId,
+			graph->leaveErrorAction( actionOrd[i], actions[i].action,
 					actions[i].localErrKey );
 			break;
 		case at_start_finish_local_error:
-			graph->startErrorAction( actionOrd[i], actions[i].action->actionId,
+			graph->startErrorAction( actionOrd[i], actions[i].action,
 					actions[i].localErrKey );
-			graph->finishErrorAction( actionOrd[i], actions[i].action->actionId,
+			graph->finishErrorAction( actionOrd[i], actions[i].action,
 					actions[i].localErrKey );
 			afterOpMinimize( graph );
 			break;
 		case at_finish_leave_local_error:
-			graph->finishErrorAction( actionOrd[i], actions[i].action->actionId,
+			graph->finishErrorAction( actionOrd[i], actions[i].action,
 					actions[i].localErrKey );
-			graph->leaveErrorAction( actionOrd[i], actions[i].action->actionId,
+			graph->leaveErrorAction( actionOrd[i], actions[i].action,
 					actions[i].localErrKey );
 			break;
 
-		default:
-			/* Parser Prevents this case. */
+		/* EOF actions. */
+		case at_start_eof:
+			graph->startEOFAction( actionOrd[i], actions[i].action );
+			afterOpMinimize( graph );
 			break;
+		case at_all_eof:
+			graph->allEOFAction( actionOrd[i], actions[i].action );
+			break;
+		case at_finish_eof:
+			graph->finishEOFAction( actionOrd[i], actions[i].action );
+			break;
+		case at_leave_eof:
+			graph->leaveEOFAction( actionOrd[i], actions[i].action );
+			break;
+		case at_start_finish_eof:
+			graph->startEOFAction( actionOrd[i], actions[i].action );
+			graph->finishEOFAction( actionOrd[i], actions[i].action );
+			afterOpMinimize( graph );
+			break;
+		case at_finish_leave_eof:
+			graph->finishEOFAction( actionOrd[i], actions[i].action );
+			graph->leaveEOFAction( actionOrd[i], actions[i].action );
+			break;
+
+		/* Remaining cases, prevented by the parser. */
+		default: break;
 		}
 	}
 }
@@ -577,7 +884,8 @@ FsmAp *FactorWithAug::walk( ParseData *pd )
 	for ( int i = 0; i < actions.length(); i++ ) {
 		if ( actions[i].type == at_start || 
 				actions[i].type == at_start_gbl_error ||
-				actions[i].type == at_start_local_error )
+				actions[i].type == at_start_local_error ||
+				actions[i].type == at_start_eof )
 			actionOrd[i] = pd->curActionOrd++;
 	}
 
@@ -588,7 +896,8 @@ FsmAp *FactorWithAug::walk( ParseData *pd )
 	for ( int i = 0; i < actions.length(); i++ ) {
 		if ( actions[i].type != at_start && 
 				actions[i].type != at_start_gbl_error &&
-				actions[i].type != at_start_local_error )
+				actions[i].type != at_start_local_error &&
+				actions[i].type != at_start_eof )
 			actionOrd[i] = pd->curActionOrd++;
 	}
 
@@ -1032,6 +1341,7 @@ FactorWithNeg::~FactorWithNeg()
 {
 	switch ( type ) {
 		case NegateType:
+		case CharNegateType:
 			delete factorWithNeg;
 			break;
 		case FactorType:
@@ -1057,6 +1367,17 @@ FsmAp *FactorWithNeg::walk( ParseData *pd )
 		afterOpMinimize( retFsm );
 		break;
 	}
+	case CharNegateType: {
+		/* Evaluate the factorWithNeg. */
+		FsmAp *toNegate = factorWithNeg->walk( pd );
+
+		/* CharNegation is subtract from dot. */
+		retFsm = new FsmAp( &pd->keyOps );
+		retFsm->dotFsm( );
+		retFsm->subtractOp( toNegate );
+		afterOpMinimize( retFsm );
+		break;
+	}
 	case FactorType: {
 		/* Evaluate the Factor. Pass it up. */
 		retFsm = factor->walk( pd );
@@ -1069,6 +1390,7 @@ void FactorWithNeg::makeNameTree( ParseData *pd )
 {
 	switch ( type ) {
 	case NegateType:
+	case CharNegateType:
 		factorWithNeg->makeNameTree( pd );
 		break;
 	case FactorType:
@@ -1081,6 +1403,7 @@ void FactorWithNeg::resolveNameRefs( ParseData *pd )
 {
 	switch ( type ) {
 	case NegateType:
+	case CharNegateType:
 		factorWithNeg->resolveNameRefs( pd );
 		break;
 	case FactorType:
@@ -1110,6 +1433,9 @@ Factor::~Factor()
 		case ParenType:
 			delete join;
 			break;
+		case LongestMatchType:
+			delete longestMatch;
+			break;
 	}
 }
 
@@ -1136,6 +1462,9 @@ FsmAp *Factor::walk( ParseData *pd )
 	case ParenType:
 		rtnVal = join->walk( pd );
 		break;
+	case LongestMatchType:
+		rtnVal = longestMatch->walk( pd );
+		break;
 	}
 	return rtnVal;
 }
@@ -1154,6 +1483,9 @@ void Factor::makeNameTree( ParseData *pd )
 	case ParenType:
 		join->makeNameTree( pd );
 		break;
+	case LongestMatchType:
+		longestMatch->makeNameTree( pd );
+		break;
 	}
 }
 
@@ -1171,6 +1503,9 @@ void Factor::resolveNameRefs( ParseData *pd )
 	case ParenType:
 		join->resolveNameRefs( pd );
 		break;
+	case LongestMatchType:
+		longestMatch->resolveNameRefs( pd );
+		break;
 	}
 }
 
@@ -1235,12 +1570,12 @@ FsmAp *Literal::walk( ParseData *pd )
 	}
 	case LitString: {
 		/* Make the array of keys in int format. */
-		long *arr = new long[str.length()];
-		makeFsmKeyArray( arr, str, str.length(), pd );
+		long *arr = new long[strlen(str)];
+		makeFsmKeyArray( arr, str, strlen(str), pd );
 
 		/* Make the new machine. */
 		rtnVal = new FsmAp( &pd->keyOps );
-		rtnVal->concatFsm( arr, str.length() );
+		rtnVal->concatFsm( arr, strlen(str) );
 		delete[] arr;
 		break;
 	}}
