@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2004 Adrian Thurston <adriant@ragel.ca>
+ *  Copyright 2001-2004 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -48,7 +48,76 @@ struct VarDef
 	Join *join;
 };
 
-/* Data. */
+
+/*
+ * LongestMatch
+ *
+ * Wherever possible the item match will execute on the character. If not
+ * possible the item match will execute on a lookahead character and either
+ * hold the current char (if one away) or backup.
+ *
+ * How to handle the problem of backing up over a buffer break?
+ * 
+ * Don't want to use pending out transitions for embedding item match because
+ * the role of item match action is different: it may sometimes match on the
+ * final transition, or may match on a lookahead character.
+ *
+ * Don't want to invent a new operator just for this. So just trail action
+ * after machine, this means we can only use literal actions.
+ *
+ * The item action may 
+ *
+ * What states of the machine will be final. The item actions that wrap around
+ * on the last character will go straight to the start state.
+ *
+ * Some transitions will be lookahead transitions, they will hold the current
+ * character. Crossing them with regular transitions must be restricted
+ * because it does not make sense. The transition cannot simultaneously hold
+ * and consume the current character.
+ */
+struct LongestMatchPart
+{
+	LongestMatchPart( Join *join, Action *action, int longestMatchId ) : 
+		join(join), action(action), longestMatchId(longestMatchId),
+		inLmSelect(false) { }
+	
+	Join *join;
+	Action *action;
+	Action *setActId;
+	Action *actRestartImm;
+	Action *thisErrAction;
+	Action *thisErrActionNoTokEnd;
+	int longestMatchId;
+	bool inLmSelect;
+
+	LongestMatchPart *prev, *next;
+};
+
+/* Declare a new type so that ptreetypes.h need not include dlist.h. */
+struct LongestMatchList : DList<LongestMatchPart> {};
+
+struct LongestMatch
+{
+	/* Construct with a list of joins */
+	LongestMatch( const InputLoc &loc, LongestMatchList *longestMatchList )
+		: loc(loc), longestMatchList(longestMatchList) { }
+
+	/* Tree traversal. */
+	FsmAp *walk( ParseData *pd );
+	void makeNameTree( ParseData *pd );
+	void resolveNameRefs( ParseData *pd );
+	void runLonestMatch( ParseData *pd, FsmAp *graph, int tokEndActionOrd );
+	void runLonestMatch( ParseData *pd, FsmAp *graph );
+	void makeActions( ParseData *pd );
+
+	Action *lmActSelect, *tokEndAction;
+
+	InputLoc loc;
+	LongestMatchList *longestMatchList;
+};
+
+
+/* List of Expressions. */
 typedef DList<Expression> ExprList;
 
 /*
@@ -111,6 +180,7 @@ struct Expression
 	Term *term;
 	BuiltinMachine builtin;
 	Type type;
+
 	Expression *prev, *next;
 };
 
@@ -235,11 +305,12 @@ struct FactorWithNeg
 {
 	enum Type { 
 		NegateType, 
+		CharNegateType,
 		FactorType
 	};
 
-	FactorWithNeg( const InputLoc &loc, FactorWithNeg *factorWithNeg ) :
-		loc(loc), factorWithNeg(factorWithNeg), factor(0), type(NegateType) { }
+	FactorWithNeg( const InputLoc &loc, FactorWithNeg *factorWithNeg, Type type) :
+		loc(loc), factorWithNeg(factorWithNeg), factor(0), type(type) { }
 
 	FactorWithNeg( const InputLoc &loc, Factor *factor ) :
 		loc(loc), factorWithNeg(0), factor(factor), type(FactorType) { }
@@ -270,6 +341,7 @@ struct Factor
 		RegExprType, 
 		ReferenceType,
 		ParenType,
+		LongestMatchType,
 	}; 
 
 	/* Construct with a literal fsm. */
@@ -292,9 +364,13 @@ struct Factor
 	Factor( const InputLoc &loc, VarDef *varDef ) :
 		loc(loc), varDef(varDef), type(ReferenceType) {}
 
-	/* Construct with a join. */
+	/* Construct with a parenthesized join. */
 	Factor( Join *join ) :
 		join(join), type(ParenType) {}
+	
+	/* Construct with a longest match operator. */
+	Factor( LongestMatch *longestMatch ) :
+		longestMatch(longestMatch), type(LongestMatchType) {}
 
 	/* Cleanup. */
 	~Factor();
@@ -311,6 +387,7 @@ struct Factor
 	RegExpr *regExp;
 	VarDef *varDef;
 	Join *join;
+	LongestMatch *longestMatch;
 	int lower, upper;
 	Type type;
 };
@@ -333,14 +410,14 @@ struct Literal
 {
 	enum LiteralType { Number, LitString };
 
-	Literal( const InputLoc &loc, String str, LiteralType type )
+	Literal( const InputLoc &loc, char *str, LiteralType type )
 		: loc(loc), str(str), type(type) { }
 
 	long makeRangeEndPoint( ParseData *pd );
 	FsmAp *walk( ParseData *pd );
 	
 	InputLoc loc;
-	String str;
+	char *str;
 	LiteralType type;
 };
 
@@ -433,27 +510,37 @@ struct InlineItem
 	enum Type 
 	{
 		Text, Goto, Call, Ret, PChar,
-		Char, Hold, Stack, Curs, Targs,
+		Char, Hold, Curs, Targs,
 		Entry, GotoE, CallE, Next, NextE,
-		Exec, Buf, BufLen, Node
+		Exec, Buf, BufLen, LmSwitch, LmAct, Node
 	};
 
-	InlineItem( char *data, Type type )
-		: data(data), nameRef(0), children(0), type(type) { }
+	InlineItem( const InputLoc &loc, char *data, Type type ) : 
+		loc(loc), data(data), nameRef(0), children(0), type(type) { }
 
-	InlineItem( NameRef *nameRef, Type type )
-		: data(0), nameRef(nameRef), children(0), type(type) { }
+	InlineItem( const InputLoc &loc, NameRef *nameRef, Type type ) : 
+		loc(loc), data(0), nameRef(nameRef), children(0), type(type) { }
 
-	InlineItem( Type type )
-		: data(0), nameRef(0), children(0), type(type) { }
+	InlineItem( const InputLoc &loc, LongestMatch *longestMatch, Type type ) : 
+		loc(loc), data(0), nameRef(0), children(0), 
+		longestMatch(longestMatch), type(type) { }
+
+	InlineItem( const InputLoc &loc, LongestMatchPart *longestMatchPart, Type type ) : 
+		loc(loc), data(0), nameRef(0), children(0), 
+		longestMatchPart(longestMatchPart), type(type) { }
+
+	InlineItem( const InputLoc &loc, Type type ) : 
+		loc(loc), data(0), nameRef(0), children(0), type(type) { }
 
 	InputLoc loc;
 	char *data;
 	NameRef *nameRef;
 	NameInst *nameTarg;
 	InlineList *children;
-
+	LongestMatch *longestMatch;
+	LongestMatchPart *longestMatchPart;
 	Type type;
+
 	InlineItem *prev, *next;
 };
 
