@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2003 Adrian Thurston <adriant@ragel.ca>
+ *  Copyright 2001-2003 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -81,7 +81,6 @@ extern CodeStyleEnum codeStyle;
 extern FILE *yyin;
 
 /* IO filenames and stream. */
-extern char *inputFile;
 extern char *outputFile;
 extern std::ostream *outStream;
 extern output_filter *outFilter;
@@ -94,7 +93,6 @@ extern bool machineSpecFound;
 extern bool graphvizDone;
 
 void yyerror( char *error );
-int yylex();
 int yyparse();
 
 extern int gblErrorCount;
@@ -111,4 +109,7 @@ std::ostream &warning( );
 std::ostream &warning( const InputLoc &loc ); 
 std::ostream &warning( int first_line, int first_column );
 
+/* Size of the include stack. */
+#define INCLUDE_STACK_SIZE 32
+
 #endif /* _RAGEL_H */
