@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2003 Adrian Thurston <adriant@ragel.ca>
+ *  Copyright 2001-2003, 2005 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -24,13 +24,6 @@
 #include "parsetree.h"
 #include "redfsm.h"
 
-/* Pass init data to base class. */
-TabCodeGen::TabCodeGen( char *fsmName, ParseData *parseData, 
-		RedFsmAp *redFsm, std::ostream &out )
-:
-	FsmCodeGen(fsmName, parseData, redFsm, out)
-{ }
-
 /* Determine if we should use indicies or not. */
 void TabCodeGen::calcIndexSize()
 {
@@ -102,14 +95,10 @@ std::ostream &TabCodeGen::ACTION_SWITCH()
 	for ( ActionList::Iter act = parseData->actionList; act.lte(); act++ ) {
 		/* Write out referenced actions. */
 		if ( act->numTransRefs > 0 || act->numEofRefs > 0 ) {
-			/* Get the function data. Print the case label.  */
+			/* Write the case label, the action and the case break. */
 			out << "\tcase " << act->actionId << ":\n";
-
-			/* Write the preprocessor line info for going into the source file. */
-			out << "#line " << act->loc.line << " \""; LDIR_PATH(inputFile) << "\"\n";
-
-			/* Write the block and close it off. */
-			out << "\t{"; INLINE_LIST(act->inlineList, 0, false) << "} break;\n";
+			ACTION( act, 0, false );
+			out << "\tbreak;\n";
 		}
 	}
 
@@ -488,12 +477,6 @@ std::ostream &TabCodeGen::LOCATE_TRANS()
 	return out;
 }
 
-std::ostream &TabCodeGen::STACK( int size )
-{
-	out << "int _top, _st[" << size << "];";
-	return out;
-}
-
 std::ostream &TabCodeGen::GOTO( NameInst *name, bool inFinish )
 {
 	/* Lookup the target. Always guaranteed to return just one target. */
@@ -544,6 +527,29 @@ std::ostream &TabCodeGen::NEXTE( InlineItem *ilItem, bool inFinish )
 	return out;
 }
 
+std::ostream &TabCodeGen::CALL( NameInst *name, int targState, bool inFinish )
+{
+	/* Lookup the target. Always guaranteed to return just one target. */
+	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
+	out << "{"; SELF() << "->stack["; SELF() << "->top++] = _cs; _cs = " << 
+			targ->value->id << "; goto again;}";
+	return out;
+}
+
+std::ostream &TabCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
+{
+	out << "{"; SELF() << "->stack["; SELF() << "->top++] = _cs; _cs = (";
+	INLINE_LIST( ilItem->children, targState, inFinish );
+	out << "); goto again;}";
+	return out;
+}
+
+std::ostream &TabCodeGen::RET( bool inFinish )
+{
+	out << "{_cs = "; SELF() << "->stack[--"; SELF() << "->top]; goto again;}";
+	return out;
+}
+
 std::ostream &TabCodeGen::CONTEXT( int contextId )
 {
 	/* Make a vect to fill with context indexed by state. */
@@ -673,32 +679,9 @@ std::ostream &TabCodeGen::STATE_DATA()
 CTabCodeGen::CTabCodeGen( char *fsmName, ParseData *parseData, 
 		RedFsmAp *redFsm, std::ostream &out )
 :
-	TabCodeGen(fsmName, parseData, redFsm, out)
+	FsmCodeGen(fsmName, parseData, redFsm, out)
 { }
 
-std::ostream &CTabCodeGen::CALL( NameInst *name, int targState, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "{fsm->_st[fsm->_top++] = _cs; _cs = " << 
-			targ->value->id << "; goto again;}";
-	return out;
-}
-
-std::ostream &CTabCodeGen::RET( bool inFinish )
-{
-	out << "{_cs = fsm->_st[--fsm->_top]; goto again;}";
-	return out;
-}
-
-std::ostream &CTabCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
-{
-	out << "{fsm->_st[fsm->_top++] = _cs; _cs = (";
-	INLINE_LIST( ilItem->children, targState, inFinish );
-	out << "); goto again;}";
-	return out;
-}
-
 void CTabCodeGen::writeOutCode()
 {
 	/* Write out state array data. */
@@ -712,7 +695,7 @@ void CTabCodeGen::writeOutCode()
 	
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
-		out << "	fsm->_top = 0;\n";
+		out << "	fsm->top = 0;\n";
 
 	INIT_CODE() <<
 		"	if ( fsm->curs >= "; FIRST_FINAL_STATE() << " )\n"
@@ -870,33 +853,9 @@ void CTabCodeGen::writeOutCode()
 CppTabCodeGen::CppTabCodeGen( char *fsmName, ParseData *parseData, 
 		RedFsmAp *redFsm, std::ostream &out ) 
 : 
-	TabCodeGen(fsmName, parseData, redFsm, out) 
+	FsmCodeGen(fsmName, parseData, redFsm, out) 
 { }
 
-std::ostream &CppTabCodeGen::CALL( NameInst *name, int targState, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "{this->_st[this->_top++] = _cs; _cs = " << 
-			targ->value->id << "; goto again;}";
-	return out;
-}
-
-std::ostream &CppTabCodeGen::RET( bool inFinish )
-{
-	out << "{_cs = this->_st[--this->_top]; goto again;}";
-	return out;
-}
-
-std::ostream &CppTabCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	out << "{this->_st[this->_top++] = _cs; _cs = (";
-	INLINE_LIST( ilItem->children, targState, inFinish );
-	out << "); goto again;}";
-	return out;
-}
-
 void CppTabCodeGen::writeOutCode()
 {
 	/* Write out state array data. */
@@ -910,7 +869,7 @@ void CppTabCodeGen::writeOutCode()
 	
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
-		out << "	this->_top = 0;\n";
+		out << "	this->top = 0;\n";
 
 	INIT_CODE() <<
 		"	if ( this->curs >= "; FIRST_FINAL_STATE() << " )\n"
@@ -1067,48 +1026,22 @@ void CppTabCodeGen::writeOutCode()
 ObjCTabCodeGen::ObjCTabCodeGen( char *fsmName, ParseData *parseData, 
 		RedFsmAp *redFsm, std::ostream &out ) 
 : 
-	TabCodeGen(fsmName, parseData, redFsm, out) 
+	FsmCodeGen(fsmName, parseData, redFsm, out) 
 { }
 
-std::ostream &ObjCTabCodeGen::CALL( NameInst *name, int targState, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	RedEntryMapEl *targ = redFsm->entryMap.find( name->id );
-	out << "{self->_st[self->_top++] = _cs; _cs = " << 
-			targ->value->id << "; goto again;}";
-	return out;
-}
-
-std::ostream &ObjCTabCodeGen::RET( bool inFinish )
-{
-	out << "{_cs = self->_st[--self->_top]; goto again;}";
-	return out;
-}
-
-std::ostream &ObjCTabCodeGen::CALLE( InlineItem *ilItem, int targState, bool inFinish )
-{
-	/* Lookup the target. Always guaranteed to return just one target. */
-	out << "{self->_st[self->_top++] = _cs; _cs = (";
-	INLINE_LIST( ilItem->children, targState, inFinish );
-	out << "); goto again;}";
-	return out;
-}
-
 void ObjCTabCodeGen::writeOutCode()
 {
 	/* Write out state array data. */
 	STATE_DATA();
 
 	out <<
-		"@implementation "; FSM_NAME() << "\n\n"
-
 		"- (int) initFsm;\n"
 		"{\n"
 		"\tself->curs = "; FSM_NAME() << "_start;\n";
 
 	/* If there are any calls, then the stack top needs initialization. */
 	if ( anyActionCalls() || anyActionRets() )
-		out << "\tself->_top = 0;\n\n";
+		out << "\tself->top = 0;\n\n";
 
 	INIT_CODE() <<
 		"\treturn ( self->curs >= "; FIRST_FINAL_STATE() << " ) ? 1 : 0;\n"
@@ -1250,7 +1183,4 @@ void ObjCTabCodeGen::writeOutCode()
 		CONTEXT( ctx->value->id );
 		out << "};\n\n";
 	}
-	
-	out <<
-		"@end\n\n";
 }
