@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001, 2002 Adrian Thurston <adriant@ragel.ca>
+ *  Copyright 2001, 2002, 2005 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -34,11 +34,9 @@ struct RedStateAp;
 /*
  * TabCodeGen
  */
-class TabCodeGen : public FsmCodeGen
+class TabCodeGen : virtual public FsmCodeGen
 {
 public:
-	TabCodeGen( char *fsmName, ParseData *parseData, 
-			RedFsmAp *redFsm, std::ostream &out );
 	virtual ~TabCodeGen() { }
 
 protected:
@@ -59,7 +57,6 @@ protected:
 	std::ostream &BSEARCH();
 	std::ostream &RANGE_BSEARCH();
 	std::ostream &LOCATE_TRANS();
-	std::ostream &STACK( int size );
 	std::ostream &GOTO( NameInst *name, bool inFinish );
 	std::ostream &GOTOE( InlineItem *ilItem, bool inFinish );
 	std::ostream &CURS( bool inFinish );
@@ -67,6 +64,9 @@ protected:
 	std::ostream &GET_ENTRY( NameInst *name, bool inFinish );
 	std::ostream &NEXT( NameInst *name, bool inFinish );
 	std::ostream &NEXTE( InlineItem *ilItem, bool inFinish );
+	std::ostream &CALL( NameInst *name, int targState, bool inFinish );
+	std::ostream &RET( bool inFinish );
+	std::ostream &CALLE( InlineItem *ilItem, int targState, bool inFinish );
 	std::ostream &CONTEXT( int contextId );
 	std::ostream &STATE_DATA();
 
@@ -79,17 +79,12 @@ protected:
 /*
  * CTabCodeGen
  */
-class CTabCodeGen : public TabCodeGen
+class CTabCodeGen : public TabCodeGen, public CCodeGen
 {
 public:
 	CTabCodeGen( char *fsmName, ParseData *parseData, 
 			RedFsmAp *redFsm, std::ostream &out );
 
-	std::ostream &CALL( NameInst *name, int targState, bool inFinish );
-	std::ostream &RET( bool inFinish );
-	std::ostream &CALLE( InlineItem *ilItem, int targState, bool inFinish );
-
-	virtual void writeOutHeader() { C_HEADER(); }
 	virtual void writeOutCode();
 };
 
@@ -97,17 +92,12 @@ public:
 /*
  * CppTabCodeGen
  */
-class CppTabCodeGen : public TabCodeGen
+class CppTabCodeGen : public TabCodeGen, public CppCodeGen
 {
 public:
 	CppTabCodeGen( char *fsmName, ParseData *parseData, 
 			RedFsmAp *redFsm, std::ostream &out );
 
-	std::ostream &CALL( NameInst *name, int targState, bool inFinish );
-	std::ostream &RET( bool inFinish );
-	std::ostream &CALLE( InlineItem *ilItem, int targState, bool inFinish );
-
-	virtual void writeOutHeader() { CPP_HEADER(); }
 	virtual void writeOutCode();
 };
 
@@ -115,17 +105,12 @@ public:
 /*
  * ObjCTabCodeGen
  */
-class ObjCTabCodeGen : public TabCodeGen
+class ObjCTabCodeGen : public TabCodeGen, public ObjCCodeGen
 {
 public:
 	ObjCTabCodeGen( char *fsmName, ParseData *parseData, 
 			RedFsmAp *redFsm, std::ostream &out );
 
-	std::ostream &CALL( NameInst *name, int targState, bool inFinish );
-	std::ostream &RET( bool inFinish );
-	std::ostream &CALLE( InlineItem *ilItem, int targState, bool inFinish );
-
-	virtual void writeOutHeader() { OBJC_HEADER(); }
 	virtual void writeOutCode();
 };
 
